
#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

List categories for a category.


A simple display for category terms
"""
import argparse
import os
import sys
import time

from rich import box, print
from rich.console import Console
from rich.prompt import Prompt
from rich.table import Table
from rich.text import Text
from rich.console import Console
from rich.markdown import Markdown


from pyegeria import (
    EgeriaTech,
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
    NO_ELEMENTS_FOUND
)

disable_ssl_warnings = True

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get("KAFKA_ENDPOINT", "localhost:9092")
EGERIA_PLATFORM_URL = os.environ.get("EGERIA_PLATFORM_URL", "https://localhost:9443")
EGERIA_VIEW_SERVER = os.environ.get("EGERIA_VIEW_SERVER", "view-server")
EGERIA_VIEW_SERVER_URL = os.environ.get(
    "EGERIA_VIEW_SERVER_URL", "https://localhost:9443"
)
EGERIA_INTEGRATION_DAEMON = os.environ.get("EGERIA_INTEGRATION_DAEMON", "integration-daemon")
EGERIA_ADMIN_USER = os.environ.get("ADMIN_USER", "garygeeke")
EGERIA_ADMIN_PASSWORD = os.environ.get("ADMIN_PASSWORD", "secret")
EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")
EGERIA_JUPYTER = bool(os.environ.get("EGERIA_JUPYTER", "False"))
EGERIA_WIDTH = int(os.environ.get("EGERIA_WIDTH", "150"))
EGERIA_category_PATH = os.environ.get("EGERIA_category_PATH", None)
EGERIA_ROOT_PATH = os.environ.get("EGERIA_ROOT_PATH", "../../")
EGERIA_INBOX_PATH = os.environ.get("EGERIA_INBOX_PATH", "md_processing/dr_egeria_inbox")
EGERIA_OUTBOX_PATH = os.environ.get("EGERIA_OUTBOX_PATH", "md_processing/dr_egeria_outbox")

console = Console(width=EGERIA_WIDTH)


def display_data_obj(
    object_type: str,
    search_string: str = "*",
    view_server: str = EGERIA_VIEW_SERVER,
    view_url: str = EGERIA_VIEW_SERVER_URL,
    user: str = EGERIA_USER,
    user_pass: str = EGERIA_USER_PASSWORD,
    jupyter: bool = EGERIA_JUPYTER,
    width: int = EGERIA_WIDTH,
    output_format: str = "TABLE",
):
    """Display either a specified category or all categories if the search_string is '*'.
    Parameters
    ----------
    object_type: str
        The type of object to display,
    search_string : str, default is '*'
        The string used to search for categories.
    view_server : str
        The view server name or address where the Egeria services are hosted.
    view_url : str
        The URL of the platform the view server is on.
    user : str
        The user ID for authentication with the Egeria server.
    user_pass : str
        The password for authentication with the Egeria server.
    jupyter : bool, optional
        A boolean indicating whether the output is intended for a Jupyter notebook (default is EGERIA_JUPYTER).
    width : int, optional
        The width of the console output (default is EGERIA_WIDTH).
    output_format: str, optional, default is 'JSON'
        One of  FORM, REPORT, TABLE
    """
    m_client = EgeriaTech(view_server, view_url, user_id=user, user_pwd=user_pass)
    token = m_client.create_egeria_bearer_token()

    match object_type:
        case "Data Structures":
            find_proc = m_client.find_data_structures
        case "Data Fields":
            find_proc = m_client.find_data_fields
        case "Data Classes":
            find_proc = m_client.find_data_classes
        case _: #catch invalid patterns
            print(f"Invalid object type: {object_type}")
            return

    try:
        if not callable(find_proc):
            raise TypeError(f"find_proc is not callable for object type '{object_type}'")

        if output_format == "FORM":
            action = "Update-Form"
        elif output_format == "REPORT":
            action = "Report"
        if output_format != "TABLE":
            file_path = os.path.join(EGERIA_ROOT_PATH, EGERIA_OUTBOX_PATH)
            file_name = f"{object_type}-{time.strftime('%Y-%m-%d-%H-%M-%S')}-{action}.md"
            full_file_path = os.path.join(file_path, file_name)
            os.makedirs(os.path.dirname(full_file_path), exist_ok=True)
            output = find_proc(search_string, output_format=output_format)
            if output == NO_ELEMENTS_FOUND:
                print(f"\n==> No elements found for search string '{search_string}'")
                return
            with open(full_file_path, 'w') as f:
                f.write(output)
            print(f"\n==> Elements output written to {full_file_path}")
            return
        else:
            output =find_proc(search_string, output_format="LIST")
            if output == NO_ELEMENTS_FOUND:
                print(f"\n==> No elements found for search string '{search_string}'")
                return
            console.print(Markdown(output))
            return




    except (InvalidParameterException, PropertyServerException) as e:
        print_exception_response(e)
    finally:
        m_client.close_session()


def main_structs():
    main(object_type = "Data Structures")

def main_fields():
    main(object_type = "Data Fields")

def main_classes():
    main(object_type = "Data Classes")

def main(object_type: str="Data Fields"):
    parser = argparse.ArgumentParser()
    parser.add_argument("--kind", choices=["Data Structures", "Data Fields", "Data Classes"])
    parser.add_argument("--server", help="Name of the server to display status for")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")

    args = parser.parse_args()

    server = args.server if args.server is not None else EGERIA_VIEW_SERVER
    url = args.url if args.url is not None else EGERIA_PLATFORM_URL
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD
    kind = args.kind if args.kind is not None else object_type

    try:
        search_string = Prompt.ask(
            "Enter the element you are searching for or '*' for all:", default="*"
        )
        output_format = Prompt.ask("What output format do you want?", choices=["TABLE", "FORM", "REPORT"],
                                   default="TABLE")

        display_data_obj(kind, search_string, server, url, userid,
                           user_pass, output_format = output_format)

    except KeyboardInterrupt:
        pass


if __name__ == "__main__":
    main()



"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.



Egeria Coco Pharmaceutical demonstration labs.

This script creates and configures the cocoMDSx server.

"""

import json

from globals import cocoCohort, cocoMDSxName, devPlatformURL, iotCohort

from pyegeria import CoreServerConfig, Platform, print_exception_response

# from pyegeria.admin_services import FullServerConfig


disable_ssl_warnings = True

mdr_server = cocoMDSxName
platform_url = devPlatformURL
admin_user = "garygeeke"
mdr_server_user_id = "cocoMDSxnpa"
mdr_server_password = "cocoMDSxpassw0rd"
metadataCollectionId = f"{mdr_server}-e915f2fa-aa3g-4396-8bde-bcd65e642b1d"
metadataCollectionName = "Development Catalog"

print("Configuring " + mdr_server + "...")

try:
    o_client = CoreServerConfig(mdr_server, platform_url, admin_user)

    o_client.set_basic_server_properties(
        "Development Server",
        "Coco Pharmaceuticals",
        platform_url,
        mdr_server_user_id,
        mdr_server_password,
        600,
    )

    # Can inherit event bus config

    # event_bus_config = {
    #     "producer": {
    #         "bootstrap.servers": "localhost:9092"
    #     },
    #     "consumer": {
    #         "bootstrap.servers": "localhost:9092"
    #     }
    # }
    #
    # o_client.set_event_bus(event_bus_config)

    security_connection_body = {
        "class": "Connection",
        "connectorType": {
            "class": "ConnectorType",
            "connectorProviderClassName": "org.odpi.openmetadata.metadatasecurity.samples.CocoPharmaServerSecurityProvider",
        },
    }

    o_client.set_server_security_connection(security_connection_body)
    o_client.add_default_log_destinations()
    o_client.set_in_mem_local_repository()
    # o_client.set_xtdb_local_kv_repository()

    o_client.set_local_metadata_collection_id(metadataCollectionId)
    o_client.set_local_metadata_collection_name(metadataCollectionName)

    o_client.add_cohort_registration(cocoCohort)
    o_client.add_cohort_registration(iotCohort)

    print(f"Configuring {mdr_server}  Access Services (OMAS)....")

    access_service_options = {
        "SupportedZones": [
            "sdlc",
            "quarantine",
            "clinical-trials",
            "research",
            "data-lake",
            "trash-can",
        ],
        "DefaultZones": ["sdlc", "quarantine"],
    }

    # o_client.configure_access_service("asset-catalog", access_service_options)
    o_client.configure_access_service("asset-consumer", access_service_options)
    o_client.configure_access_service("asset-owner", access_service_options)
    o_client.configure_access_service("community-profile", {"KarmaPointPlateau": "500"})
    # o_client.configure_access_service("glossary-view", {})
    o_client.configure_access_service("data-science", access_service_options)
    # o_client.configure_access_service("subject-area", {})
    o_client.configure_access_service("asset-manager", access_service_options)
    o_client.configure_access_service("governance-engine", access_service_options)
    o_client.configure_access_service("governance-server", access_service_options)
    # o_client.configure_access_service("data-manager", access_service_options)
    o_client.configure_access_service("it-infrastructure", access_service_options)
    o_client.configure_access_service("project-management", access_service_options)
    o_client.configure_access_service("software-tech", access_service_options)
    # o_client.configure_access_service("devops", access_service_options)
    o_client.configure_access_service("digital-architecture", access_service_options)
    o_client.configure_access_service("design-model", access_service_options)

    p_client = Platform(mdr_server, platform_url, admin_user)
    p_client.activate_server_stored_config()

    print(f"\n\n\tConfiguration of {mdr_server} is complete.")

    config = o_client.get_stored_configuration()
    print(f"The server stored configuration is \n{json.dumps(config, indent=4)}")

except Exception as e:
    print_exception_response(e)



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

A command that takes an output format set name, looks up the format set using select_report_spec,
and invokes the specified function with parameters filled from command line arguments.

This command works with any format set defined in output_formatter.py that has an "action" property.
It dynamically determines the appropriate client class and method to call based on the format set,
and prompts for any required parameters that aren't provided on the command line.

Features:
- Works with any format set that has an "action" property
- Dynamically adds command-line arguments based on the format set's required/optional parameters
- Prompts for required parameters if not provided
- Supports multiple output formats (TABLE, DICT, FORM, REPORT, HTML, LIST)
- Handles both instance methods and standalone functions
- Provides informative error messages

Examples:
    # List all collections using the Collections format set
    format_set_action --format-set Collections --search-string "*" --output-format TABLE

    # Output collections in DICT format
    format_set_action --format-set Collections --search-string "*" --output-format DICT

    # Use a specific search string
    format_set_action --format-set Collections --search-string "Data*" --output-format TABLE
    
    # Use a different format set (Data Dictionary)
    format_set_action --format-set "Data Dictionary" --search-string "*" --output-format TABLE
    
    # Use a format set with multiple parameters (will prompt for missing parameters)
    format_set_action --format-set DigitalProducts --output-format REPORT
"""
import argparse
import json
import os
import time

from loguru import logger
from pydantic import ValidationError
from rich import box
from rich.console import Console
from rich.markdown import Markdown
from rich.prompt import Prompt
from rich.table import Table
from rich.text import Text
import pydevd_pycharm

from pyegeria import (
    EgeriaTech,
    CollectionManager,
    NO_ELEMENTS_FOUND, GovernanceOfficer, GlossaryManager, print_validation_error,
)
from pyegeria.config import settings
from pyegeria.external_links import ExternalReferences
from pyegeria.logging_configuration import config_logging
from pyegeria._output_format_models import load_format_sets_from_json
from pyegeria.base_report_formats import (select_report_format, get_report_format_heading,
                                          load_user_report_specs, load_report_specs,
                                          get_report_format_description)
from pyegeria._exceptions_new import PyegeriaException, print_exception_response

# pydevd_pycharm.settrace('host.docker.internal',  # Use 'host.docker.internal' to connect to the host machine
#                              port=5678,               # Port to communicate with PyCharm
#                              stdoutToServer=True,     # Forward stdout to PyCharm
#                              stderrToServer=True,     # Forward stderr to PyCharm
#                              suspend=True)            # Suspend execution until the debugger is connected
#


EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")


app_config = settings.Environment
config_logging()

print(f"Console width is {app_config.console_width}")
console = Console(
    style="bold bright_white on black",
    width=app_config.console_width,
    force_terminal=not app_config.egeria_jupyter,
    )

def execute_format_set_action(
    format_set_name: str,
    view_server: str = app_config.egeria_view_server,
    view_url: str = app_config.egeria_view_server_url,
    user: str = settings.User_Profile.user_name,
    user_pass: str = settings.User_Profile.user_pwd,
    output_format: str = "TABLE",
    **kwargs
):
    """
    Looks up the specified output format set, extracts the function and parameters,
    and invokes the function with the parameters.

    Parameters
    ----------
    format_set_name : str
        The name of the output format set to use.
    view_server : str
        The view server name or address where the Egeria services are hosted.
    view_url : str
        The URL of the platform the view server is on.
    user : str
        The user ID for authentication with the Egeria server.
    user_pass : str
        The password for authentication with the Egeria server.
    jupyter : bool, optional
        A boolean indicating whether the output is intended for a Jupyter notebook.
    width : int, optional
        The width of the console output.
    output_format : str, optional
        Format of the output. Default is TABLE.
    **kwargs : dict
        Additional parameters to override the default parameters in the format set.
    """
    # Get the output format set
    # logger.info(f"Entering execute_format_set_action, format set name: {format_set_name}, search_string: {kwargs.get('search_string',"meow")}")
    # logger.info(json.dumps(kwargs, indent=2))
    format_set = select_report_format(format_set_name, output_format)
    if not format_set:
        print(f"Error: Output format set '{format_set_name}' does not have a format compatible with output format '{output_format}'.")
        return

    # Check if the format set has an action property
    if "action" not in format_set:
        print(f"Error: Output format set '{format_set_name}' does not have an action property.")
        return

    # Extract the function and parameters from the action property (now a dict)
    action = format_set["action"]
    func = action.get("function")
    required_params = action.get("required_params", action.get("user_params", []))
    optional_params = action.get("optional_params", [])
    spec_params = action.get("spec_params", {})
    
    # Create a params dictionary from required/optional and spec_params
    params = {}
    for param in required_params:
        if param in kwargs and kwargs[param]:
            params[param] = kwargs[param]
        elif param not in kwargs and param not in spec_params:
            print(f"Warning: Required parameter '{param}' not provided for format set '{format_set_name}'.")
    for param in optional_params:
        if param in kwargs and kwargs[param]:
            params[param] = kwargs[param]
    
    # Add spec_params to params
    params.update(spec_params)

    # Delegate execution to exec_report_spec for canonical behavior
    from pyegeria.format_set_executor import exec_report_spec
    ofmt = (output_format or "TABLE").upper()
    mapped = "DICT" if ofmt == "TABLE" else ofmt
    res = exec_report_spec(
        format_set_name,
        output_format=mapped,
        params=params,
        view_server=view_server,
        view_url=view_url,
        user=user,
        user_pass=user_pass,
    )

    # Handle TABLE rendering locally
    if ofmt == "TABLE":
        if res.get("kind") != "json":
            print("No results or unexpected response:", res)
            return
        data = res.get("data")
        # Render basic table
        table = Table(
            title=f"{format_set_name} @ {time.asctime()}",
            style="bright_white on black",
            header_style="bright_white on dark_blue",
            title_style="bold white on black",
            caption_style="white on black",
            show_lines=True,
            box=box.ROUNDED,
            caption=f"View Server '{view_server}' @ Platform - {view_url}",
            expand=True,
        )
        rows = data if isinstance(data, list) else [data]
        if rows and isinstance(rows[0], dict):
            cols = list(rows[0].keys())
            for c in cols:
                table.add_column(str(c), justify="left", style="cyan")
            for r in rows:
                table.add_row(*[str(r.get(c, "")) for c in cols])
        else:
            table.add_column("Value")
            for r in rows:
                table.add_row(str(r))
        # Optional header
        from pyegeria.base_report_formats import get_report_format_heading, get_report_format_description
        heading = get_report_format_heading(format_set_name)
        desc = get_report_format_description(format_set_name)
        if heading and desc:
            console.print(Markdown(f"# {heading}\n{desc}\n"))
        console.print(table)
        return

    # For non-table formats, write to file based on MIME/kind
    kind = res.get("kind")
    mime = res.get("mime")
    out_dir = os.path.join(app_config.pyegeria_root, app_config.dr_egeria_outbox)
    ts = time.strftime('%Y-%m-%d-%H-%M-%S')
    safe_name = "".join(c for c in format_set_name if c.isalnum() or c in ("-","_","+","."," ")).strip().replace(" ", "_")

    if kind == "text":
        content = res.get("content", "")
        ext = ".html" if mime == "text/html" else ".md"
    elif kind == "json":
        content = json.dumps(res.get("data"), indent=2)
        ext = ".json"
    else:
        content = json.dumps(res, indent=2, default=str)
        ext = ".txt"

    file_name = f"{safe_name}-{ts}{ext}"
    full_file_path = os.path.join(out_dir, file_name)
    os.makedirs(os.path.dirname(full_file_path), exist_ok=True)
    with open(full_file_path, "w", encoding="utf-8") as f:
        f.write(content)
    print(f"\n==> Output written to {full_file_path}")
    if ext == ".html":
        print(f"\n==> Web link: [{file_name}]({app_config.pyegeria_publishing_root}/{file_name})")
    return

    # Determine the appropriate client class based on the format set name or function
    client_class = None
    method_name = None
    
    # If function name is provided as a string, parse it to get class and method
    if isinstance(func, str) and "." in func:
        class_name, method_name = func.split(".")
        if class_name == "CollectionManager":
            client_class = CollectionManager
        elif class_name == "GovernanceOfficer":
            client_class = GovernanceOfficer
        elif class_name == "GlossaryManager":
            client_class = GlossaryManager
        elif class_name == "ExternalReference":
            client_class = ExternalReferences
        else:
            client_class = EgeriaTech


        # Add more client classes as needed
    
    # # If client_class is still None, determine based on format set name
    # if client_class is None:
    #     if format_set_name in ["Collections", "Data Dictionary", "Data Specification", "DigitalProducts"]:
    #         client_class = CollectionManager
    #         method_name = "find_collections"
    #     else:
    #         # Default to CollectionManager for now
    #         client_class = CollectionManager
    
    # Create the client instance
    client = client_class(view_server, view_url, user_id=user, user_pwd=user_pass)
    
    # If method_name is set, get the method from the client
    # Note: We need to convert func from string to method reference even if func is already set
    if method_name:
        if hasattr(client, method_name):
            func = getattr(client, method_name)
        else:
            print(f"Error: Method '{method_name}' not found in client class '{client_class.__name__}'.")
            return
    
    # Check if we have a valid function to call
    if not func and not method_name:
        print(f"Error: No valid function found for format set '{format_set_name}'.")
        return
    
    client.create_egeria_bearer_token()

    # Get heading and description information
    heading = get_report_format_heading(format_set_name)
    desc = get_report_format_description(format_set_name)
    preamble = f"# {heading}\n{desc}\n\n" if heading and desc else ""

    try:
        # Invoke the function with the parameters
        if output_format != "TABLE":
            file_path = os.path.join(app_config.pyegeria_root, app_config.dr_egeria_outbox)
            if output_format == "HTML":
                file_name = f"{format_set_name}-{time.strftime('%Y-%m-%d-%H-%M-%S')}.html"
            elif output_format == "JSON":
                file_name = f"{format_set_name}-{time.strftime('%Y-%m-%d-%H-%M-%S')}.json"
            elif output_format == "DICT":
                file_name = f"{format_set_name}-{time.strftime('%Y-%m-%d-%H-%M-%S')}.py"
            elif output_format == "MERMAID":
                file_name = f"{format_set_name}-{time.strftime('%Y-%m-%d-%H-%M-%S')}.md"
            else:
                file_name = f"{format_set_name}-{time.strftime('%Y-%m-%d-%H-%M-%S')}.md"
            full_file_path = os.path.join(file_path, file_name)
            os.makedirs(os.path.dirname(full_file_path), exist_ok=True)
            
            # Add output_format to params
            params['output_format'] = output_format
            
            # Call the function with the parameters
            print(f"\n==> Calling function: {func} with parameters:{params}")
            try:
                if callable(func):
                    # Call the function as an instance method of the client
                    output = func(**params)
                else:
                    # For standalone functions, call with client as first argument
                    output = func(client, **params)
            except ValidationError as e:
                print_validation_error(e)
                return

            except TypeError as e:
                # Handle parameter mismatch errors
                print(f"Error calling function: {e}")
                print(f"Parameters provided: {params}")
                return
            
            if output == NO_ELEMENTS_FOUND:
                print(f"\n==> No elements found for format set '{format_set_name}'")
                return
            elif isinstance(output, (str, list)) and output_format == "DICT":
                output = json.dumps(output, indent=4)
            elif isinstance(output, (str, list)) and output_format in[ "REPORT" ]:
                output = preamble + output
            elif isinstance(output, (str, list)) and output_format == "HTML":
                pass

            with open(full_file_path, 'w') as f:
                f.write(output)
            print(f"\n==> Output written to {full_file_path}")
            if output_format == "HTML":
                print(f"\n==> Web link: [{file_name}]({app_config.pyegeria_publishing_root}/{file_name}")
            return
        else:
            # For TABLE output, add output_format to params
            params['output_format'] = "TABLE"
            print(f"\n==> Calling function: {func} with parameters:{params}")
            # Call the function and create a table
            try:
                if hasattr(client, method_name):  # It's a method of the client
                    # Call the function as an instance method of the client
                    result = func(**params)
                else:
                    # For standalone functions, call with client as first argument
                    result = func(client, **params)
            except TypeError as e:
                # Handle parameter mismatch errors
                print(f"Error calling function: {e}")
                print(f"Parameters provided: {params}")
                return

            if not result or result == NO_ELEMENTS_FOUND:
                print(f"\n==> No elements found for format set '{format_set_name}'")
                return


            if heading and desc:
                console.print(Markdown(preamble))

            # Create a table for the results
            table = Table(
                title=f"{format_set_name} @ {time.asctime()}",
                style="bright_white on black",
                header_style="bright_white on dark_blue",
                title_style="bold white on black",
                caption_style="white on black",
                show_lines=True,
                box=box.ROUNDED,
                caption=f"View Server '{view_server}' @ Platform - {view_url}",
                expand=True,
            )

            # Handle both list and dictionary results
            if isinstance(result, list):
                if not result:
                    print(f"\n==> No elements found for format set '{format_set_name}'")
                    return

                # Sort the results by display_name if available
                if result[0].get("display_name", None):
                    sorted_results = sorted(result, key=lambda k: k.get("display_name", ""))
                elif result[0].get("Display Name", None):
                    sorted_results = sorted(result, key=lambda k: k.get("Display Name", ""))
                else:
                    sorted_results = result

                # Add columns dynamically based on the first result
                column_headings = list(sorted_results[0].keys())
                for heading in column_headings:
                    if heading == "Usage":
                        table.add_column(heading, justify="left", style="cyan")
                    else:
                        table.add_column(heading, justify="left", style="cyan")

                # Add rows
                for item in sorted_results:
                    row_values = []
                    for key in column_headings:
                        value = item.get(key, "")
                        row_values.append(str(value))

                    table.add_row(*row_values)
            else:
                # Handle single dictionary result
                column_headings = list(result.keys())
                for heading in column_headings:
                    table.add_column(heading, justify="left", style="cyan")

                row_values = []
                for key in column_headings:
                    value = result.get(key, "")
                    row_values.append(str(value))

                table.add_row(*row_values)

            # Print the table

            console.print(table)

    except PyegeriaException as e:
        print_exception_response(e)
    finally:
        client.close_session()


def main():
    # First, parse just the format-set argument to determine which other arguments to add
    logger.enable("pyegeria")
    initial_parser = argparse.ArgumentParser(add_help=False)
    initial_parser.add_argument("--report", help="Name of the report spec", required=True)
    initial_args, _ = initial_parser.parse_known_args()

    # Get the format set to determine parameters
    format_set_name = initial_args.report
    format_set = select_report_format(format_set_name, "ANY")

    # Check if the format set exists
    if not format_set:
        print(f"Error: Report Spec for '{format_set_name}' not found.")
        print("Available Report Specs:")
        from pyegeria.base_report_formats import report_spec_list
        for name in report_spec_list():
            print(f"  - {name}")
        return
    
    # Create the full parser with all arguments
    parser = argparse.ArgumentParser(description="Execute an action from Report Spec")
    parser.add_argument("--report", help="Name of the report spec", required=True)
    parser.add_argument("--server", help="Name of the server to connect to")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")
    parser.add_argument("--output-format", help="Output format (TABLE, DICT, FORM, REPORT, HTML, LIST)", 
                        choices=["TABLE", "DICT", "FORM", "REPORT", "HTML", "LIST", "MERMAID"], default="TABLE")



    # Add arguments based on the format set's required/optional params
    required_params = []
    optional_params = []
    if "action" in format_set:
        action = format_set["action"]
        required_params = action.get("required_params", action.get("user_params", []))
        optional_params = action.get("optional_params", [])
        
        for param in sorted(set(required_params + optional_params)):
            parser.add_argument(f"--{param.replace('_', '-')}", help=f"{param.replace('_', ' ')} parameter")
    else:
        print(f"Error: Report Spec '{format_set_name}' does not have an action property.")
        return
    
    args = parser.parse_args()
    app_settings = settings
    app_config = app_settings.Environment

    server = args.server if args.server is not None else app_config.egeria_view_server
    url = args.url if args.url is not None else app_config.egeria_view_server_url
    print(f"root path: {app_config.pyegeria_root}, config_file: {app_config.pyegeria_config_file}")
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD
    format_set_name = args.report
    output_format = args.output_format

    logger.info(f"view server @ {url}")
    # Collect all parameters from args
    kwargs = {}
    for param in sorted(set(required_params + optional_params)):
        arg_name = param.replace('_', '-')
        if hasattr(args, arg_name) and getattr(args, arg_name) is not None:
            kwargs[param] = getattr(args, arg_name)
        elif hasattr(args, param) and getattr(args, param) is not None:
            kwargs[param] = getattr(args, param)
    
    try:
        # If a required parameter is not provided, prompt for it
        for param in required_params:
            if param not in kwargs:
                default_value = "*" if param == "search_string" else ""
                prompt_text = f"Enter {param.replace('_', ' ')}:"
                if default_value:
                    prompt_text += f" (default: {default_value})"
                value = Prompt.ask(prompt_text, default=default_value).strip()
                kwargs[param] = value
        print(f"Using Report Spec {format_set_name} and output format {output_format} with parameters: {kwargs} ")
        execute_format_set_action(
            format_set_name=format_set_name,
            view_server=server,
            view_url=url,
            user=userid,
            user_pass=user_pass,
            output_format=output_format,
            **kwargs
        )

    except KeyboardInterrupt:
        pass

    except PyegeriaException as e:
        print_exception_response(e)


if __name__ == "__main__":
    main()


#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

List certification types


A simple display for certification types
"""
import argparse
import os
import sys
import time

from rich import box
from rich.console import Console
from rich.table import Table

from pyegeria import (
    ClassificationManager,
    PyegeriaException, settings
)
app_config = settings.Environment

EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")

disable_ssl_warnings = True


def display_certifications(
    server: str,
    url: str,
    username: str,
    user_password: str,
    time_out: int = 60,
    jupyter: bool = app_config.egeria_jupyter,
    width: int = app_config.console_width,
):
    console = Console(width=width, force_terminal=not jupyter, soft_wrap=True)

    g_client = ClassificationManager(
        server, url, user_id=username, user_pwd=user_password
    )
    token = g_client.create_egeria_bearer_token(username, user_password)

    def generate_table(search_string: str = "CertificationType") -> Table:
        """Make a new table."""
        table = Table(
            title=f"Certifications Types  @ {time.asctime()}",
            header_style="white on dark_blue",
            style="bold white on black",
            row_styles=["bold white on black"],
            title_style="bold white on black",
            caption_style="white on black",
            show_lines=True,
            box=box.ROUNDED,
            caption=f"View Server '{server}' @ Platform - {url}",
            expand=True,
        )
        table.add_column("Name", max_width=15)
        table.add_column("summary")
        table.add_column("domainID")
        table.add_column("Unique Name")
        table.add_column("Scope")
        # table.add_column("Qualified Name",max_width=15)
        table.add_column("Description")
        table.add_column("Details")
        table.add_column("Related Elements")

        certs = g_client.get_elements(search_string, page_size=100, time_out=time_out)
        if type(certs) is str:
            return table

        for element in certs:
            properties = element["properties"]
            summary = properties.get("summary", "---")
            domain = str(properties.get("domainIdentifier", "---"))
            unique_name = properties.get("qualifiedName", "---")
            scope = properties.get("scope", "---")
            description = properties.get("description", "---")
            details = properties.get("details", "---")
            name = properties.get("displayName", "---")
            cert_guid = element["elementHeader"]["guid"]

            related = g_client.get_related_elements(cert_guid)
            if (len(related) > 0) and (type(related) is list):
                rel_md = ""
                for rel in related:
                    rel_type = rel["relationshipHeader"]["type"]["typeName"]
                    rel_element_props = rel["relatedElement"]["properties"]
                    rel_el_md = f"* Rel Type: {rel_type}\n"
                    for key in rel_element_props.keys():
                        rel_el_md += f"* {key}: {rel_element_props[key]}\n"
                    rel_md += f"----\n{rel_el_md}\n"
            else:
                rel_md = "---"

            table.add_row(
                name, summary, domain, unique_name, scope, description, details, rel_md
            )

        g_client.close_session()

        return table

    try:

        with console.pager(styles=True):
            console.print(generate_table("CertificationType"), soft_wrap=True)

    except (
        PyegeriaException
    ) as e:
        console.print_exception()
        sys.exit(1)

    except ValueError as e:
        console.print(
            f"\n\n====> Invalid Search String - must be greater than four characters long"
        )
        sys.exit(1)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--server", help="Name of the server to display status for")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")
    parser.add_argument("--time_out", help="Time Out")

    args = parser.parse_args()

    server = args.server if args.server is not None else app_config.egeria_view_server
    url = args.url if args.url is not None else app_config.egeria_view_server_url
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD
    time_out = args.time_out if args.time_out is not None else 60
    try:
        display_certifications(server, url, userid, user_pass, time_out)
    except KeyboardInterrupt:
        pass


if __name__ == "__main__":
    main()



"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.



Egeria Coco Pharmaceutical demonstration labs.

This script creates and configures the cocoMDS5 - Business Systems

"""

import json

from globals import (
    adminUserId,
    cocoCohort,
    cocoMDS1Name,
    cocoMDS2Name,
    cocoMDS3Name,
    cocoMDS4Name,
    cocoMDS5Name,
    cocoMDS6Name,
    cocoMDSxName,
    cocoOLS1Name,
    cocoView1Name,
    corePlatformName,
    corePlatformURL,
    dataLakePlatformName,
    dataLakePlatformURL,
    devCohort,
    devPlatformName,
    devPlatformURL,
    iotCohort,
    max_paging_size,
)

from pyegeria import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
)
from pyegeria.core_omag_server_config import CoreServerConfig
from pyegeria.platform_services import Platform

disable_ssl_warnings = True

mdr_server = cocoMDS5Name
platform_url = corePlatformURL
admin_user = "garygeeke"
mdr_server_user_id = "cocoMDS5npa"
mdr_server_password = "cocoMDS5passw0rd"
metadataCollectionId = f"{mdr_server}-e915f2fa-aa3g-4396-8bde-bcd65e642b1d"
metadataCollectionName = "Business Systems Catalog"

print("Configuring " + mdr_server + "...")
try:
    o_client = CoreServerConfig(mdr_server, platform_url, admin_user)

    o_client.set_basic_server_properties(
        "Business Systems",
        "Coco Pharmaceuticals",
        platform_url,
        mdr_server_user_id,
        mdr_server_password,
        max_paging_size,
    )

    # Inherit event bus config

    # event_bus_config = {
    #     "producer": {
    #         "bootstrap.servers": "localhost:9092"
    #     },
    #     "consumer": {
    #         "bootstrap.servers": "localhost:9092"
    #     }
    # }
    #
    # o_client.set_event_bus(event_bus_config)

    security_connection_body = {
        "class": "Connection",
        "connectorType": {
            "class": "ConnectorType",
            "connectorProviderClassName": "org.odpi.openmetadata.metadatasecurity.samples.CocoPharmaServerSecurityProvider",
        },
    }
    o_client.set_server_security_connection(security_connection_body)
    o_client.add_default_log_destinations()

    # o_client.set_in_mem_local_repository()
    o_client.set_xtdb_local_kv_repository()

    o_client.set_local_metadata_collection_id(metadataCollectionId)
    o_client.set_local_metadata_collection_name(metadataCollectionName)

    o_client.add_cohort_registration(cocoCohort)

    proxy_details = (
        "org.odpi.openmetadata.adapters.repositoryservices.readonly.repositoryconnector."
        + "ReadOnlyOMRSRepositoryConnectorProvider"
    )
    o_client.set_repository_proxy_details(proxy_details)

    p_client = Platform(mdr_server, platform_url, admin_user)
    p_client.activate_server_stored_config()

    print(f"\n\n\tConfiguration of {mdr_server} is complete.")

    config = o_client.get_stored_configuration()
    print(f"The server stored configuration is \n{json.dumps(config, indent=4)}")

except Exception as e:
    print_exception_response(e)



"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.



This module is for testing the core OMAG config class and methods
The routines assume that pytest is being used as the test tool and framework.

A running Egeria environment is needed to run these tests.

"""

import json

import pytest
from rich import print as rprint
from rich.console import Console
from rich.table import Table

from pyegeria._exceptions import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
)
from pyegeria.registered_info import RegisteredInfo

disable_ssl_warnings = True


class TestRegisteredInfoServices:
    good_platform1_url = "https://127.0.0.1:9443"
    good_platform2_url = "https://cray.local:9443"
    bad_platform1_url = "https://localhost:9443"

    good_user_1 = "garygeeke"
    good_user_2 = "erinoverview"
    bad_user_1 = "eviledna"
    bad_user_2 = ""

    good_server_1 = "simple-metadata-store"
    good_server_2 = "qs-view-server"
    good_server_3 = "qs-metadata-store"
    bad_server_1 = "coco"
    bad_server_2 = ""

    @pytest.mark.parametrize(
        "service_kind",
        [
            # (
            #     None
            # ),
            # (
            #     "all"
            # ),
            # (
            #     "access-services"
            # ),
            # (
            #     "common-services"
            # ),
            # (
            #     "engine-services"
            #
            # ),
            # (
            #     "governance-services"
            # ),
            ("integration-services"),
            # (
            #         "view-services"
            # ),
        ],
    )
    def test_list_registered_svcs(self, service_kind):
        try:
            r_client = RegisteredInfo(
                self.good_server_1, self.good_platform1_url, self.good_user_1
            )
            service_kind = "all"
            response = r_client.list_registered_svcs(service_kind)

            assert (type(response) is list) or (
                type(response) is str
            ), "No services found"
            if type(response) is list:
                print(f"\n\nView services configuration for {service_kind}: \n\n")
                # print(json.dumps(response, indent=4))
                rprint(response)
            else:
                print(f"\n\n{response}: \n\n")

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

    def test_list_severity_definitions(self):
        try:
            r_client = RegisteredInfo(
                self.good_server_3, self.good_platform1_url, user_id=self.good_user_1
            )

            response = r_client.list_severity_definitions()
            console = Console()

            table = Table(
                title="Severity Codes",
                style="black on grey66",
                header_style="white on dark_blue",
            )
            table.add_column("Number")
            table.add_column("Name", width=30)
            table.add_column("Description")

            for code in response:
                ordinal, name, description = (
                    str(code["ordinal"]),
                    code["name"],
                    code["description"],
                )
                table.add_row(ordinal, name, description)
            console.print(table)
            assert True

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"

    def test_list_asset_types(self):
        user = self.good_user_2
        try:
            r_client = RegisteredInfo(
                self.good_server_2, self.good_platform1_url, user, "secret"
            )
            token = r_client.create_egeria_bearer_token(user, "secret")
            response = r_client.list_asset_types()

            assert type(response) is list, "No services found"
            if type(response) is list:
                print(f"\n\nAsset types are: \n\n")
                print(json.dumps(response, indent=4))
            else:
                print(f"\n\n{response}: \n\n")

        except (
            InvalidParameterException,
            PropertyServerException,
            UserNotAuthorizedException,
        ) as e:
            print_exception_response(e)
            assert False, "Invalid request"



"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

A pure helper to execute a FormatSet action and return normalized results without side effects
(printing or writing to disk). This is suitable for programmatic use (e.g., MCP adapters,
notebooks, services) that want DICT/JSON or text output directly.

Notes
- Only use this for read-style format sets. It assumes the action is safe to call.
- Credentials and endpoints default from pyegeria.config.settings (ENV/config.json/ENV files).
- Prefer output_format="DICT" for machine-consumable results. REPORT/HTML/MERMAID are returned as text.
"""
from __future__ import annotations

import asyncio
import json
import sys
from typing import Any, Dict, Optional

from loguru import logger

from pyegeria.egeria_tech_client import EgeriaTech
from pyegeria.collection_manager import CollectionManager
from pyegeria.governance_officer import GovernanceOfficer
from pyegeria.glossary_manager import GlossaryManager
from pyegeria._globals import NO_ELEMENTS_FOUND
from pyegeria.config import settings
from pyegeria.external_links import ExternalReferences
from pyegeria._exceptions_new import PyegeriaException
from pyegeria.base_report_formats import (
    select_report_spec,
    get_report_spec_heading,
    get_report_spec_description,
)


_CLIENT_CLASS_MAP = {
    "CollectionManager": CollectionManager,
    "GovernanceOfficer": GovernanceOfficer,
    "GlossaryManager": GlossaryManager,
    "ExternalReference": ExternalReferences,
}


def _resolve_client_and_method(func_decl: str):
    """Given a function declaration like 'ClassName.method', return (client_class, method_name)."""
    if not isinstance(func_decl, str) or "." not in func_decl:
        return (EgeriaTech, None)
    class_name, method_name = func_decl.split(".", 1)
    client_class = _CLIENT_CLASS_MAP.get(class_name, EgeriaTech)
    return (client_class, method_name)



async def safe_call_tool(func, **call_params):
    """
    Safely calls a function, awaiting it only if it is an asynchronous coroutine.
    """
    try:
        if asyncio.iscoroutinefunction(func):
            # If it's an async function, call it with await
            print("DEBUG: Function is async, using await.", file=sys.stderr)
            result = await func(**call_params)
        else:
            # If it's a synchronous function, call it directly
            print("DEBUG: Function is sync, calling directly.", file=sys.stderr)
            result = func(**call_params)

        return result

    except Exception as e:
        print(f"ERROR calling function: {e}", file=sys.stderr)
        raise



async def _async_run_report(
    report_name: str,
    egeria_client: EgeriaTech,
    output_format: str = "DICT",
    params: Optional[Dict[str, Any]] = None,
) -> Dict[str, Any]:
    """
        Execute the action for a given format set and return a normalized result structure.

        Returns shapes:
        - {"kind":"empty"}
        - {"kind":"json","data": <list|dict|any>}
        - {"kind":"text","mime": "text/markdown"|"text/html","content": str}
        - {"kind":"unknown","raw": any}
        """
    params = dict(params or {})
    user_name = egeria_client.user_id
    user_pwd = egeria_client.user_pwd

    # Resolve the format set and action
    effective_format = "REPORT" if output_format == "HTML" else output_format
    fmt = select_report_spec(report_name, effective_format)
    if not fmt:
        raise ValueError(
            f"Output format set '{report_name}' does not have a compatible '{output_format}' format."
        )
    if "action" not in fmt:
        raise ValueError(f"Output format set '{report_name}' does not have an action property.")

    action = fmt["action"]
    func_decl = action.get("function")
    if isinstance(func_decl, str) and "." in func_decl:
        class_name, method_name = func_decl.split(".")

    if not method_name.startswith("_async_"):
        method_name = "_async_" + method_name
        func_decl = class_name + "." + method_name

    required_params = action.get("required_params", action.get("user_params", [])) or []
    optional_params = action.get("optional_params", []) or []
    spec_params = action.get("spec_params", {}) or {}
    print(f"func_decl={func_decl}", file=sys.stderr)
    # Build call params: required/optional provided by caller + fixed spec_params
    call_params: Dict[str, Any] = {}

    # Populate required and optional params when provided
    for p in required_params:
        if p in params and params[p] is not None:
            call_params[p] = params[p]
        elif p not in spec_params:
            # Missing required param
            logger.warning(f"Required parameter '{p}' not provided for format set '{report_name}'.")
    for p in optional_params:
        if p in params and params[p] is not None:
            call_params[p] = params[p]

    # Include fixed specifics
    call_params.update(spec_params)

    # Always include output_format and report_spec for downstream rendering
    call_params["output_format"] = output_format
    call_params["report_spec"] = report_name

    client_class, method_name = _resolve_client_and_method(func_decl)


    try:
        func = getattr(egeria_client, method_name) if method_name and hasattr(egeria_client, method_name) else None
        # Add logging to validate func
        msg = f"DEBUG: func={func}, method_name={method_name}, client_class={client_class}"
        logger.debug(msg)
        print(msg, file=sys.stderr)

        if func is None or not callable(func):
            raise TypeError(
                f"Resolved function '{method_name}'  not found in client class '{client_class.__name__}' is not callable."
            )
        # Only (re)create a bearer token if one is not already set on the client.
        try:
            existing_token = None
            if hasattr(egeria_client, "get_token"):
                existing_token = egeria_client.get_token()
            if not existing_token:
                print("DEBUG: No existing bearer token; attempting async creation...", file=sys.stderr)
                if user_name and user_pwd:
                    await egeria_client._async_create_egeria_bearer_token(user_name, user_pwd)
                else:
                    print("DEBUG: Missing credentials; skipping token creation and relying on pre-initialized token.", file=sys.stderr)
            else:
                print("DEBUG: Using existing bearer token.", file=sys.stderr)
        except Exception as auth_err:
            # Do not fail the entire call if token refresh fails; downstream call may still work
            print(f"DEBUG: Token creation/lookup issue: {auth_err}", file=sys.stderr)
        result = await func(**call_params)

        if not result or result == NO_ELEMENTS_FOUND:
            return {"kind": "empty"}

        # Prepare optional preamble for narrative outputs
        heading = get_report_spec_heading(report_name)
        desc = get_report_spec_description(report_name)
        preamble = f"# {heading}\n{desc}\n\n" if heading and desc else ""

        if output_format in {"DICT", "JSON", "ALL"}:
            # Return raw data (list/dict/any) — do not stringify here
            return {"kind": "json", "data": result}
        elif output_format in {"REPORT", "MD", "FORM"}:
            # These are markdown narratives, pass-through as markdown text
            content = preamble + (str(result) if not isinstance(result, (list, dict)) else ("```json\n" + json.dumps(result, indent=2) + "\n```"))
            return {"kind": "text", "mime": "text/markdown", "content": content}
        elif output_format == "LIST":
            # Produce a Markdown table from structured data
            def to_rows(res):
                if isinstance(res, list):
                    return res
                elif isinstance(res, dict):
                    return [res]
                else:
                    return [res]
            rows = to_rows(result)
            # Normalize rows into list of dicts
            norm_rows = []
            has_dict = False
            for r in rows:
                if isinstance(r, dict):
                    norm_rows.append(r)
                    has_dict = True
                else:
                    norm_rows.append({"Value": r})
            # Determine columns
            cols = []
            if has_dict:
                for r in norm_rows:
                    if isinstance(r, dict):
                        for k in r.keys():
                            if k not in cols:
                                cols.append(str(k))
            else:
                cols = ["Value"]
            # Build markdown table
            header = "| " + " | ".join(cols) + " |"
            separator = "| " + " | ".join(["---"] * len(cols)) + " |"
            lines = []
            if preamble:
                lines.append(preamble.strip())
            lines.append(header)
            lines.append(separator)
            for r in norm_rows:
                row_vals = [str(r.get(c, "")) for c in cols]
                lines.append("| " + " | ".join(row_vals) + " |")
            content = "\n".join(lines)
            return {"kind": "text", "mime": "text/markdown", "content": content}
        elif output_format == "MERMAID":
            text = str(result)
            fenced = text if "```mermaid" in text else f"{preamble}```mermaid\n{text}\n```"
            return {"kind": "text", "mime": "text/markdown", "content": fenced}
        elif output_format == "HTML":
            # HTML should be REPORT transformed to HTML
            from pyegeria.output_formatter import markdown_to_html
            # If backend returned non-string, render JSON for safety
            md_source = preamble + (str(result) if not isinstance(result, (list, dict)) else ("```json\n" + json.dumps(result, indent=2) + "\n```"))
            content = markdown_to_html(md_source)
            return {"kind": "text", "mime": "text/html", "content": content}
        else:
            # Unknown or table-like formats which aren't appropriate for MCP by default
            return {"kind": "unknown", "raw": result}

    except PyegeriaException as e:
        # Re-raise with a simpler message for upstream mapping
        raise



def exec_report_spec(
    format_set_name: str | dict,
    *,
    output_format: str = "DICT",
    params: Optional[Dict[str, Any]] = None,
    view_server: str = settings.Environment.egeria_view_server,
    view_url: str = settings.Environment.egeria_view_server_url,
    user: str = settings.User_Profile.user_name,
    user_pass: str = settings.User_Profile.user_pwd,
) -> Dict[str, Any]:
    """
    Execute the action for a given format set and return a normalized result structure.

    Returns shapes:
    - {"kind":"empty"}
    - {"kind":"json","data": <list|dict|any>}
    - {"kind":"text","mime": "text/markdown"|"text/html","content": str}
    - {"kind":"unknown","raw": any}
    """
    output_format = (output_format or "DICT").upper()
    params = dict(params or {})

    # Resolve the format set and action
    if isinstance(format_set_name, dict):
        fmt = format_set_name
    else:
        fmt = select_report_spec(format_set_name, output_format)
    if not fmt:
        raise ValueError(
            f"Output report spec '{format_set_name}' does not have a compatible '{output_format}' format."
        )
    if "action" not in fmt:
        raise ValueError(f"Output report spec '{format_set_name}' does not have an action property.")

    action = fmt["action"]
    func_decl = action.get("function")
    required_params = action.get("required_params", action.get("user_params", [])) or []
    optional_params = action.get("optional_params", []) or []
    spec_params = action.get("spec_params", {}) or {}

    # Build call params: required/optional provided by caller + fixed spec_params
    call_params: Dict[str, Any] = {}

    # Populate required and optional params when provided
    for p in required_params:
        if p in params and params[p] is not None:
            call_params[p] = params[p]
        elif p not in spec_params:
            # Missing required param
            logger.warning(f"Required parameter '{p}' not provided for report spec '{format_set_name}'.")
    for p in optional_params:
        if p in params and params[p] is not None:
            call_params[p] = params[p]

    # Include fixed specifics
    call_params.update(spec_params)

    # Always include output_format and report_spec for downstream rendering
    call_params["output_format"] = output_format
    call_params["report_spec"] = format_set_name

    client_class, method_name = _resolve_client_and_method(func_decl)
    client = client_class(view_server, view_url, user_id=user, user_pwd=user_pass)

    try:
        client.create_egeria_bearer_token()
        func = getattr(client, method_name) if method_name and hasattr(client, method_name) else None
        if func is None:
            raise AttributeError(
                f"Method '{method_name}' not found in client class '{client_class.__name__}'."
            )

        result = func(**call_params)

        if not result or result == NO_ELEMENTS_FOUND:
            return {"kind": "empty"}

        # Prepare optional preamble for narrative outputs
        heading = get_report_spec_heading(format_set_name)
        desc = get_report_spec_description(format_set_name)
        preamble = f"# {heading}\n{desc}\n\n" if heading and desc else ""

        if output_format in {"DICT", "JSON", "ALL"}:
            # Return raw data (list/dict/any) — do not stringify here
            return {"kind": "json", "data": result}
        elif output_format in {"REPORT", "MD", "FORM"}:
            # These are markdown narratives, pass-through as markdown text
            content = preamble + (str(result) if not isinstance(result, (list, dict)) else ("```json\n" + json.dumps(result, indent=2) + "\n```"))
            return {"kind": "text", "mime": "text/markdown", "content": content}
        elif output_format == "LIST":
            # Produce a simple markdown list from structured data when possible
            if isinstance(result, (list, dict)):
                rows = result if isinstance(result, list) else [result]
                def flatten(item):
                    if isinstance(item, dict):
                        return item
                    return {"value": item}
                flat_rows = [flatten(r) for r in rows]
                keys = []
                for r in flat_rows:
                    for k in r.keys():
                        if k not in keys:
                            keys.append(k)
                label_key = keys[0] if keys else "value"
                lines = []
                if preamble:
                    lines.append(preamble.strip())
                lines += [f"- {str(r.get(label_key, ''))}" for r in flat_rows]
                content = "\n".join(lines)
            else:
                content = preamble + str(result)
            return {"kind": "text", "mime": "text/markdown", "content": content}
        elif output_format == "MERMAID":
            text = str(result)
            fenced = text if "```mermaid" in text else f"{preamble}```mermaid\n{text}\n```"
            return {"kind": "text", "mime": "text/markdown", "content": fenced}
        elif output_format == "HTML":
            from pyegeria.output_formatter import markdown_to_html
            md_source = preamble + (str(result) if not isinstance(result, (list, dict)) else ("```json\n" + json.dumps(result, indent=2) + "\n```"))
            content = markdown_to_html(md_source)
            return {"kind": "text", "mime": "text/html", "content": content}
        else:
            # Unknown or table-like formats which aren't appropriate for MCP by default
            return {"kind": "unknown", "raw": result}

    except PyegeriaException as e:
        # Re-raise with a simpler message for upstream mapping
        raise
    finally:
        try:
            client.close_session()
        except Exception:
            pass



"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.


This file contains a set of test routines to test the platform_services of the Egeria python client.
The routines assume that pytest is being used as the test tool and framework. These tests are not idempotent - much
more work would be needed to ensure that they work correctly in any order. That said, running the last test,
`test_platform_services` does a fairly good job of exercising the test cases in the right sequence - and so is a
good way to start.

A running Egeria environment is needed to run these tests. A set of platform, server and user variables are
created local to the TestPlatform class to hold the set of values to be used for testing. The default values have
been configured based on pre-built startup Egeria configurations and the local testing environment. The pre-built
configurations are documented at
`https://github.com/odpi/egeria/blob/main/open-metadata-resources/open-metadata-deployment/sample-configs/README.md`.

However, the tests are not dependent on this configuration. It should, however, be noted that the tests are currently
order sensitive - in other words if you delete all the servers the subsequent tests that expect the servers to be
available may fail..

"""

import json
import time
from contextlib import nullcontext as does_not_raise
from time import sleep

import pytest

disable_ssl_warnings = True


from rich.live import Live
from rich.table import Table

from pyegeria._exceptions_new import (
    PyegeriaException,
    PyegeriaAPIException,
    PyegeriaUnauthorizedException,
    print_exception_response,
    print_basic_exception
)
from pyegeria.platform_services import Platform
from pyegeria.server_operations import ServerOps


class TestPlatform:
    good_platform1_url = "https://127.0.0.1:9443"
    good_platform2_url = "https://localhost:8443"
    good_platform3_url = "https://egeria.pdr-associates.com:9443"
    bad_platform1_url = "https://localhost:9443"

    # good_platform1_url = "https://127.0.0.1:30080"
    # good_platform2_url = "https://127.0.0.1:30081"
    # bad_platform1_url = "https://localhost:9443"

    good_user_1 = "garygeeke"
    good_user_2 = "erinoverview"
    bad_user_1 = "eviledna"
    bad_user_2 = ""

    good_server_1 = "active-metadata-store"
    good_server_2 = "qs-metadata-store"
    good_server_3 = "view-server"
    good_server_4 = "engine-host"
    bad_server_1 = "coco"
    bad_server_2 = ""

    @pytest.mark.parametrize(
        "server, url, user_id, exc_type, expectation",
        [
            # (
            #         "meow",
            #         "https://google.com",
            #         "garygeeke",
            #         "PyegeriaException",
            #         pytest.raises(PyegeriaException),
            # ),
            (
                "active-metadata-server",
                "https://localhost:9443",
                "garygeeke",
                "secret",
                does_not_raise(),
            ),
            # (
            #         "cocoMDS1",
            #         good_platform3_url,
            #         "garygeeke",
            #         "",
            #         does_not_raise(),
            # ),
            # (
            #         "cocoMDS2",
            #         "https://127.0.0.1:9443",
            #         None,
            #         "PyegeriaException",
            #         pytest.raises(PyegeriaException),
            # ),
            # (
            #         "cocoMDS2",
            #         "https://127.0.0.1:9443/open-metadata/admin-services/users/garygeeke/servers/active-metadata-store",
            #         "meow",
            #         "PyegeriaException",
            #         pytest.raises(PyegeriaException),
            # ),
            # ("", "", "", "PyegeriaException", pytest.raises(PyegeriaException)),
        ],
    )
    def test_get_platform_origin(self, server, url, user_id, exc_type, expectation):
        with expectation as excinfo:
            p_client = Platform(server, url, user_id)
            response_text = p_client.get_platform_origin()
            if response_text is not None:
                print("\n\n" + response_text)
                assert True
        if excinfo:
            print_exception_response(excinfo.value)
            assert excinfo.typename is exc_type, "Unexpected exception"

    def test_shutdown_platform(self, server: str = good_server_1):
        try:
            p_client = Platform(server, self.good_platform1_url, self.good_user_1)
            p_client.shutdown_platform()
            print(f"\n\n\t Platform shutdown")
            assert True

        except (
            PyegeriaException,
            PyegeriaAPIException,
            PyegeriaUnauthorizedException,
        ) as e:
            print_basic_exception(e)
            assert e.related_http_code is not "200", "Invalid parameters"
        finally:
            p_client.close_session()

    def test_activate_server_stored_config(self, server: str = "active-metadata-store"):
        """
        Need to decide if its worth it to broaden out the test cases..for instance
        in this method if there is an exception - such as invalid server name
        then the test case fails because the response is used before set..

        """
        try:
            p_client = Platform(server, self.good_platform1_url, self.good_user_1)
            p_client.activate_server_stored_config(server)
            print(f"\n\n\t server {server} configured and activated successfully")
            assert True

        except (
            PyegeriaException,
            PyegeriaAPIException,
            PyegeriaUnauthorizedException,
        ) as e:
            print_exception_response(e)
            assert e.related_http_code == "200", "Invalid parameters"

        finally:
            p_client.close_session()

    def test_shutdown_server(self, server: str = "cocoMDS1"):
        try:
            p_client = Platform(server, self.good_platform1_url, self.good_user_1)
            p_client.shutdown_server(server)
            print(f"\n\n\t server {server} was shut down successfully")
            assert True

        except (
            PyegeriaException,
            PyegeriaAPIException,
            PyegeriaUnauthorizedException,
        ) as e:
            print_exception_response(e)
            assert e.related_http_code == 404, "Invalid parameters"
        finally:
            p_client.close_session()

    def test_get_known_servers(self, server: str = good_server_3):
        try:
            p_client = Platform(server, self.good_platform2_url, self.good_user_1)
            response = p_client.get_known_servers()
            print(f"\n\n\t response = {response}")
            assert len(response) > 0, "Empty server list"

        except (
            PyegeriaException,
            PyegeriaAPIException,
            PyegeriaUnauthorizedException,
        ) as e:
            print_exception_response(e)
            assert e.related_http_code == "200", "Invalid parameters"
        finally:
            p_client.close_session()

    def test_shutdown_unregister_servers(self, server: str = good_server_2):
        try:
            p_client = Platform(server, self.good_platform1_url, self.good_user_1)
            p_client.shutdown_unregister_servers()
            print(
                f"\n\n\t Servers on platform {p_client.platform_url} shutdown and unregistered"
            )
            assert True

        except (PyegeriaException, PyegeriaAPIException) as e:
            print_exception_response(e)
            assert e.related_http_code == "200", "Invalid parameters"
        finally:
            p_client.close_session()

    @pytest.mark.parametrize(
        "server, url, user_id, ex_type, expectation",
        [
            (
                "meow",
                "https://google.com",
                "garygeeke",
                "PyegeriaException",
                pytest.raises(PyegeriaException),
            ),
            (
                "cocoMDS2",
                "https://localhost:9448",
                "garygeeke",
                "PyegeriaException",
                pytest.raises(PyegeriaException),
            ),
            (
                "cocoMDS1",
                "https://127.0.0.1:30081",
                "garygeeke",
                "PyegeriaException",
                pytest.raises(PyegeriaException),
            ),
            (
                "cocoMDS9",
                "https://127.0.0.1:9443",
                "garygeeke",
                "PyegeriaException",
                does_not_raise(),
            ),
            (
                good_server_2,
                "https://127.0.0.1:9443",
                "garygeeke",
                None,
                does_not_raise(),
            ),
            (
                "cocoMDS2",
                "https://127.0.0.1:9443/open-metadata/admin-services/users/garygeeke/servers/active-metadata-store",
                "meow",
                "PyegeriaUnauthorizedException",
                pytest.raises(PyegeriaUnauthorizedException),
            ),
        ],
    )
    def test_get_active_configuration(self, server, url, user_id, ex_type, expectation):
        with Platform(server, url, user_id) as p_client:
            with expectation as excinfo:
                response = p_client.get_active_configuration(server)
                print(
                    f"\n\n\tThe active configuration of {server} is \n{json.dumps(response, indent=4)}"
                )
                assert True

            if excinfo:
                print_exception_response(excinfo.value)
                assert excinfo.typename is ex_type

    def test_activate_server_supplied_config(self, server: str = good_server_3):
        server_name = server
        config_body = {
            "class": "OMAGServerConfig",
            "versionId": "V2.0",
            "localServerId": "f81bbe7a-2592-4afe-bd851-64dfb3bd7920",
            "localServerName": "test-store",
            "localServerURL": "https://localhost:9443",
            "localServerUserId": "activemdsnpa",
            "maxPageSize": 1000,
            "eventBusConfig": {
                "class": "EventBusConfig",
                "topicURLRoot": "egeria.omag",
                "configurationProperties": {
                    "producer": {"bootstrap.servers": "localhost:9092"},
                    "consumer": {"bootstrap.servers": "localhost:9092"},
                },
            },
            "accessServicesConfig": [
                {
                    "class": "AccessServiceConfig",
                    "accessServiceId": 210,
                    "accessServiceDevelopmentStatus": "STABLE",
                    "accessServiceAdminClass": "org.odpi.openmetadata.accessservices.datamanager.admin.DataManagerAdmin",
                    "accessServiceName": "Data Manager",
                    "accessServiceFullName": "Data Manager OMAS",
                    "accessServiceURLMarker": "data-manager",
                    "accessServiceDescription": "Capture changes to the data stores and data set managed by a data manager such as a database server, content manager or file system.",
                    "accessServiceWiki": "https://egeria-project.org/services/omas/data-manager/overview/",
                    "accessServiceOperationalStatus": "ENABLED",
                    "accessServiceOutTopic": {
                        "class": "Connection",
                        "headerVersion": 0,
                        "displayName": "Kafka Event Bus Connection",
                        "connectorType": {
                            "class": "ConnectorType",
                            "headerVersion": 0,
                            "connectorProviderClassName": "org.odpi.openmetadata.adapters.eventbus.topic.kafka.KafkaOpenMetadataTopicProvider",
                        },
                        "endpoint": {
                            "class": "Endpoint",
                            "headerVersion": 0,
                            "address": "egeria.omag.server.test-store.omas.datamanager.outTopic",
                        },
                        "configurationProperties": {
                            "producer": {"bootstrap.servers": "localhost:9092"},
                            "local.server.id": "f81bbe7a-2592-4afe-bd81-64dfb3bd7920",
                            "consumer": {"bootstrap.servers": "localhost:9092"},
                        },
                    },
                },
                {
                    "class": "AccessServiceConfig",
                    "accessServiceId": 204,
                    "accessServiceDevelopmentStatus": "STABLE",
                    "accessServiceAdminClass": "org.odpi.openmetadata.accessservices.assetmanager.admin.AssetManagerAdmin",
                    "accessServiceName": "Asset Manager",
                    "accessServiceFullName": "Asset Manager OMAS",
                    "accessServiceURLMarker": "asset-manager",
                    "accessServiceDescription": "Manage metadata from a third party asset manager",
                    "accessServiceWiki": "https://egeria-project.org/services/omas/asset-manager/overview/",
                    "accessServiceOperationalStatus": "ENABLED",
                    "accessServiceOutTopic": {
                        "class": "Connection",
                        "headerVersion": 0,
                        "displayName": "Kafka Event Bus Connection",
                        "connectorType": {
                            "class": "ConnectorType",
                            "headerVersion": 0,
                            "connectorProviderClassName": "org.odpi.openmetadata.adapters.eventbus.topic.kafka.KafkaOpenMetadataTopicProvider",
                        },
                        "endpoint": {
                            "class": "Endpoint",
                            "headerVersion": 0,
                            "address": "egeria.omag.server.test-store.omas.assetmanager.outTopic",
                        },
                        "configurationProperties": {
                            "producer": {"bootstrap.servers": "localhost:9092"},
                            "local.server.id": "f81bbe7a-2592-4afe-bd81-64dfb3bd7920",
                            "consumer": {"bootstrap.servers": "localhost:9092"},
                        },
                    },
                },
                {
                    "class": "AccessServiceConfig",
                    "accessServiceId": 207,
                    "accessServiceDevelopmentStatus": "STABLE",
                    "accessServiceAdminClass": "org.odpi.openmetadata.accessservices.communityprofile.admin.CommunityProfileAdmin",
                    "accessServiceName": "Community Profile",
                    "accessServiceFullName": "Community Profile OMAS",
                    "accessServiceURLMarker": "community-profile",
                    "accessServiceDescription": "Define personal profile and collaborate.",
                    "accessServiceWiki": "https://egeria-project.org/services/omas/community-profile/overview/",
                    "accessServiceOperationalStatus": "ENABLED",
                    "accessServiceOutTopic": {
                        "class": "Connection",
                        "headerVersion": 0,
                        "displayName": "Kafka Event Bus Connection",
                        "connectorType": {
                            "class": "ConnectorType",
                            "headerVersion": 0,
                            "connectorProviderClassName": "org.odpi.openmetadata.adapters.eventbus.topic.kafka.KafkaOpenMetadataTopicProvider",
                        },
                        "endpoint": {
                            "class": "Endpoint",
                            "headerVersion": 0,
                            "address": "egeria.omag.server.test-store.omas.communityprofile.outTopic",
                        },
                        "configurationProperties": {
                            "producer": {"bootstrap.servers": "localhost:9092"},
                            "local.server.id": "f81bbe7a-2592-4afe-bd81-64dfb3bd7920",
                            "consumer": {"bootstrap.servers": "localhost:9092"},
                        },
                    },
                },
                {
                    "class": "AccessServiceConfig",
                    "accessServiceId": 227,
                    "accessServiceDevelopmentStatus": "STABLE",
                    "accessServiceAdminClass": "org.odpi.openmetadata.accessservices.governanceserver.admin.GovernanceServerAdmin",
                    "accessServiceName": "Governance Server",
                    "accessServiceFullName": "Governance Server OMAS",
                    "accessServiceURLMarker": "governance-server",
                    "accessServiceDescription": "Supply the governance engine definitions to the engine hosts and the and integration group definitions to the integration daemons.",
                    "accessServiceWiki": "https://egeria-project.org/services/omas/governance-server/overview/",
                    "accessServiceOperationalStatus": "ENABLED",
                    "accessServiceOutTopic": {
                        "class": "Connection",
                        "headerVersion": 0,
                        "displayName": "Kafka Event Bus Connection",
                        "connectorType": {
                            "class": "ConnectorType",
                            "headerVersion": 0,
                            "connectorProviderClassName": "org.odpi.openmetadata.adapters.eventbus.topic.kafka.KafkaOpenMetadataTopicProvider",
                        },
                        "endpoint": {
                            "class": "Endpoint",
                            "headerVersion": 0,
                            "address": "egeria.omag.server.test-store.omas.governanceserver.outTopic",
                        },
                        "configurationProperties": {
                            "producer": {"bootstrap.servers": "localhost:9092"},
                            "local.server.id": "f81bbe7a-2592-4afe-bd81-64dfb3bd7920",
                            "consumer": {"bootstrap.servers": "localhost:9092"},
                        },
                    },
                },
                {
                    "class": "AccessServiceConfig",
                    "accessServiceId": 219,
                    "accessServiceDevelopmentStatus": "STABLE",
                    "accessServiceAdminClass": "org.odpi.openmetadata.accessservices.governanceengine.admin.GovernanceEngineAdmin",
                    "accessServiceName": "Governance Engine",
                    "accessServiceFullName": "Governance Engine OMAS",
                    "accessServiceURLMarker": "governance-engine",
                    "accessServiceDescription": "Provide metadata services and watch dog notification to the governance action services.",
                    "accessServiceWiki": "https://egeria-project.org/services/omas/governance-engine/overview/",
                    "accessServiceOperationalStatus": "ENABLED",
                    "accessServiceOutTopic": {
                        "class": "Connection",
                        "headerVersion": 0,
                        "displayName": "Kafka Event Bus Connection",
                        "connectorType": {
                            "class": "ConnectorType",
                            "headerVersion": 0,
                            "connectorProviderClassName": "org.odpi.openmetadata.adapters.eventbus.topic.kafka.KafkaOpenMetadataTopicProvider",
                        },
                        "endpoint": {
                            "class": "Endpoint",
                            "headerVersion": 0,
                            "address": "egeria.omag.server.test-store.omas.governanceengine.outTopic",
                        },
                        "configurationProperties": {
                            "producer": {"bootstrap.servers": "localhost:9092"},
                            "local.server.id": "f81bbe7a-2592-4afe-bd81-64dfb3bd7920",
                            "consumer": {"bootstrap.servers": "localhost:9092"},
                        },
                    },
                },
            ],
            "repositoryServicesConfig": {
                "class": "RepositoryServicesConfig",
                "auditLogConnections": [
                    {
                        "class": "Connection",
                        "headerVersion": 0,
                        "qualifiedName": "Console- default",
                        "displayName": "Console",
                        "connectorType": {
                            "class": "ConnectorType",
                            "headerVersion": 0,
                            "connectorProviderClassName": "org.odpi.openmetadata.adapters.repositoryservices.auditlogstore.console.ConsoleAuditLogStoreProvider",
                        },
                        "configurationProperties": {
                            "supportedSeverities": [
                                "Unknown",
                                "Information",
                                "Decision",
                                "Action",
                                "Error",
                                "Exception",
                                "Security",
                                "Startup",
                                "Shutdown",
                                "Asset",
                                "Cohort",
                            ]
                        },
                    }
                ],
                "localRepositoryConfig": {
                    "class": "LocalRepositoryConfig",
                    "metadataCollectionId": "be62f138-62fb-4ecf-85a6-ee497d8a4a90",
                    "localRepositoryMode": "OPEN_METADATA_NATIVE",
                    "localRepositoryLocalConnection": {
                        "class": "Connection",
                        "headerVersion": 0,
                        "displayName": "Local KV XTDB Repository",
                        "connectorType": {
                            "class": "ConnectorType",
                            "headerVersion": 0,
                            "connectorProviderClassName": "org.odpi.openmetadata.adapters.repositoryservices.xtdb.repositoryconnector.XTDBOMRSRepositoryConnectorProvider",
                        },
                        "configurationProperties": {
                            "xtdbConfig": {
                                "xtdb.lucene/lucene-store": {
                                    "db-dir": "data/servers/test-store/repository/xtdb-kv/lucene"
                                },
                                "xtdb/tx-log": {
                                    "kv-store": {
                                        "db-dir": "data/servers/test-store/repository/xtdb-kv/rdb-tx",
                                        "xtdb/module": "xtdb.rocksdb/-\u003ekv-store",
                                    }
                                },
                                "xtdb/index-store": {
                                    "kv-store": {
                                        "db-dir": "data/servers/test-store/repository/xtdb-kv/rdb-index",
                                        "xtdb/module": "xtdb.rocksdb/-\u003ekv-store",
                                    }
                                },
                                "xtdb/document-store": {
                                    "kv-store": {
                                        "db-dir": "data/servers/test-store/repository/xtdb-kv/rdb-docs",
                                        "xtdb/module": "xtdb.rocksdb/-\u003ekv-store",
                                    }
                                },
                            }
                        },
                    },
                    "localRepositoryRemoteConnection": {
                        "class": "Connection",
                        "headerVersion": 0,
                        "displayName": "Local Repository Remote Connection",
                        "connectorType": {
                            "class": "ConnectorType",
                            "headerVersion": 0,
                            "connectorProviderClassName": "org.odpi.openmetadata.adapters.repositoryservices.rest.repositoryconnector.OMRSRESTRepositoryConnectorProvider",
                        },
                        "endpoint": {
                            "class": "Endpoint",
                            "headerVersion": 0,
                            "address": "https://localhost:9443/servers/test-store",
                        },
                    },
                    "eventsToSaveRule": "ALL",
                    "eventsToSendRule": "ALL",
                },
                "enterpriseAccessConfig": {
                    "class": "EnterpriseAccessConfig",
                    "enterpriseMetadataCollectionName": "test-store Enterprise Metadata Collection",
                    "enterpriseMetadataCollectionId": "3d6db963-2c87-455d-b4de-d7862ba5fc9b",
                    "enterpriseOMRSTopicConnection": {
                        "class": "VirtualConnection",
                        "headerVersion": 0,
                        "displayName": "Enterprise OMRS Topic Connection",
                        "connectorType": {
                            "class": "ConnectorType",
                            "headerVersion": 0,
                            "connectorProviderClassName": "org.odpi.openmetadata.repositoryservices.connectors.omrstopic.OMRSTopicProvider",
                        },
                        "embeddedConnections": [
                            {
                                "class": "EmbeddedConnection",
                                "headerVersion": 0,
                                "position": 0,
                                "displayName": "Enterprise OMRS Events",
                                "embeddedConnection": {
                                    "class": "Connection",
                                    "headerVersion": 0,
                                    "displayName": "Kafka Event Bus Connection",
                                    "connectorType": {
                                        "class": "ConnectorType",
                                        "headerVersion": 0,
                                        "connectorProviderClassName": "org.odpi.openmetadata.adapters.eventbus.topic.inmemory.InMemoryOpenMetadataTopicProvider",
                                    },
                                    "endpoint": {
                                        "class": "Endpoint",
                                        "headerVersion": 0,
                                        "address": "test-store.openmetadata.repositoryservices.enterprise.test-store.OMRSTopic",
                                    },
                                    "configurationProperties": {
                                        "local.server.id": "f81bbe7a-2592-4afe-bd81-64dfb3bd7920",
                                        "eventDirection": "inOut",
                                    },
                                },
                            }
                        ],
                    },
                    "enterpriseOMRSTopicProtocolVersion": "V1",
                },
            },
        }

        try:
            p_client = Platform(server_name, self.good_platform1_url, self.good_user_1)
            p_client.activate_server_supplied_config(config_body, server)
            print(f"\n\n\tServer {server_name} now configured and activated")
            assert True
        except (PyegeriaException, PyegeriaAPIException) as e:
            print_exception_response(e)
            assert e.related_http_code != "404", "Invalid parameters"

        finally:
            p_client.close_session()

    def test_get_active_server_instance_status(self, server: str = good_server_2):
        try:
            p_client = Platform(server, self.good_platform1_url, self.good_user_1)
            response = p_client.get_active_server_instance_status()
            if type(response) is str:
                print("Server instance status indicates: " + response)
            else:
                print(f"\n\n\tActive server status: {json.dumps(response, indent=4)}")
            assert True

        except (PyegeriaException, PyegeriaAPIException) as e:
            print_exception_response(e)
            assert e.related_http_code != "404", "Invalid parameters"

        finally:
            p_client.close_session()

    def test_is_server_known(self, server: str = good_server_2):
        try:
            p_client = Platform(server, self.good_platform1_url, self.good_user_1)
            response = p_client.is_server_known(server)
            print(f"\n\n\tis_server_known() for server {server} reports {response}")
            assert (response is True) or (response is False), "Exception happened?"

        except (PyegeriaException, PyegeriaAPIException) as e:
            print_exception_response(e)
            assert e.related_http_code == "200", "Invalid parameters"

        finally:
            p_client.close_session()

    def test_is_server_configured(self, server: str = good_server_3):
        try:
            p_client = Platform(server, self.good_platform1_url, self.good_user_1)
            response = p_client.is_server_configured(server)
            print(
                f"\n\n\tis_server_configured() for server {server} reports {response}"
            )
            assert (response is True) or (response is False), "Exception happened?"

        except (PyegeriaException, PyegeriaAPIException) as e:
            print_exception_response(e)
            assert e.related_http_code == "200", "Invalid parameters"
        finally:
            p_client.close_session()

    def test_get_active_server_list(self, server: str = good_server_2):
        try:
            p_client = Platform(server, self.good_platform1_url, self.good_user_1)
            response = p_client.get_active_server_list()
            print(f"\n\n\tThe active servers are: {response}")
            assert len(response) > 0, "Exception?"

        except (PyegeriaException, PyegeriaAPIException) as e:
            print_exception_response(e)
            assert e.related_http_code != "200", "Invalid parameters"

        finally:
            p_client.close_session()

    def test_shutdown_all_servers(self):
        try:
            server = self.good_server_1
            p_client = Platform(server, self.good_platform1_url, self.good_user_1)
            p_client.shutdown_all_servers()
            print(f"\n\n\tAll servers have been shutdown")
            assert True

        except (PyegeriaException, PyegeriaAPIException) as e:
            print_exception_response(e)
            assert e.related_http_code != "200", "Invalid parameters"

        finally:
            p_client.close_session()

    def test_check_server_active(self, server: str = good_server_2):
        try:
            p_client = Platform(server, self.good_platform1_url, self.good_user_1)
            response = p_client.check_server_active(server)
            print(f"\n\nserver {server} active state is {str(response)}")
            assert response in (True, False), "Bad Response"

        except (PyegeriaException) as e:
            print_basic_exception(e)
            assert e.related_http_code != "200", "Invalid parameters"

        finally:
            p_client.close_session()

    def test_activate_server_if_down(self):
        try:
            server = self.good_server_2
            p_client = Platform(server, self.good_platform1_url, self.good_user_1)

            response = p_client.activate_server_if_down(server, verbose=True)
            print(f"\n\n\t  activation success was {response}")
            assert True

        except (
            PyegeriaException,
            PyegeriaAPIException,
            PyegeriaUnauthorizedException,
        ) as e:
            print_exception_response(e)
            assert e.related_http_code != "200", "Invalid parameters"

    def test_activate_servers_on_platform(self, server: str = good_server_2):
        server_name = server
        with Platform(
            server_name, self.good_platform1_url, self.good_user_1
        ) as p_client:
            server_list = p_client.get_known_servers()
            print(f"\n\n\tKnown servers on the platform are: {server_list}")

            print(f"\n\tTrying to activate {server_list}")
            response = p_client.activate_servers_on_platform(server_list, verbose=True)
            print(f"\n\n\t\tactivate_servers_on_platform: success = {response}")
            if response is False:
                print("\n\t\tCheck that all servers are configured")
            known_servers = p_client.get_known_servers()
            print(f"\n\n\tKnown servers on the platform are: {known_servers}")
            assert True, "Issues encountered "

        if p_client.exc_type:
            print_exception_response(p_client.exc_val)
            assert False

    def test_activate_platform(self, server: str = good_server_3):
        try:
            p_client = Platform(server, self.good_platform1_url, self.good_user_1)
            p_client.activate_platform(
                "laz", ["active-metadata-store", "simple-metadata-store"]
            )
            print(f"\n\n\tPlatform laz activated")
            assert True

        except (PyegeriaException, PyegeriaAPIException) as e:
            print_exception_response(e)
            assert e.related_http_code != "200", "Invalid parameters"

        finally:
            p_client.close_session()

    def test_display_status(self, server: str = good_server_2):
        server_name = server

        table = Table(
            title=f"Server Status for Platform - {time.asctime()}",
            style="black on grey66",
            header_style="white on dark_blue",
            show_lines=True,
            expand=True,
        )
        table.add_column("Known Server", width=20)
        table.add_column("Status", width=10)
        table.add_column("Service Name", width=20, no_wrap=True)
        table.add_column("Service Status", width=10)
        try:
            with ServerOps(
                server_name, self.good_platform1_url, self.good_user_1
            ) as p_client:
                server_list = p_client.get_known_servers()
                live = Live(table, refresh_per_second=1)
                with live:
                    for server in server_list:
                        status = "Unknown"
                        if p_client.check_server_active(server):
                            status = "Active"
                        else:
                            status = "Inactive"
                        services = p_client.get_active_server_status(server)["serverStatus"][
                            "services"
                        ]
                        for service in services:
                            service_name = service["serviceName"]
                            service_status = service["serviceStatus"]
                            table.add_row(server, status, service_name, service_status),

                    sleep(2)

        except (
            PyegeriaException,
            PyegeriaAPIException,
            PyegeriaUnauthorizedException,
        ) as e:
            print_exception_response(e)
            assert e.related_http_code != "200", "Invalid parameters"

        finally:
            p_client.close_session()

    def test_platform_services(self):
        server = self.good_server_2
        url = "https://127.0.0.1:9443"
        user_id = "garygeeke"

        # tp = TestPlatform()
        print("Running through a test scenario for the platform services module.")

        print(
            "\n\n==========================================>Check what servers are active"
        )
        self.test_get_active_server_list(server)

        print(
            "\n\n==========================================>First we will shutdown and unregister all servers"
        )
        self.test_shutdown_unregister_servers(server)

        print(
            "\n\n==========================================>Check what servers are active"
        )
        self.test_get_active_server_list(server)

        print(
            "\n\n==========================================>Now we will look to see what servers are known"
        )
        self.test_get_known_servers()

        print(
            "\n\n==========================================>Start `simple-metadata-store`"
        )
        self.test_activate_server_stored_config(server)

        print(
            "\n\n==========================================>Check that simple-metadata-store is active"
        )
        self.test_check_server_active()

        print(
            "\n\n==========================================>Retrieve the server status for simple-metadata-store"
        )
        self.test_get_active_server_instance_status()

        print(
            "\n\n==========================================>Retrieve the active_server_instance_status for simple-metadata-store"
        )
        self.test_get_active_server_instance_status()

        print(
            "\n\n==========================================>Retrieve the active service list for simple-metadata-store"
        )
        self.test_get_active_server_instance_status()

        print(
            "\n\n==========================================>Start the 'active-metadata-store'"
        )
        self.test_activate_server_stored_config("active-metadata-store")

        print(
            "\n\n==========================================>Get the list of all active servers"
        )
        self.test_get_active_server_list(server)

        print(
            "\n\n==========================================>Stop the simple-metadata-store server"
        )
        self.test_shutdown_server("simple-metadata-store")

        print(
            "\n\n==========================================>Get the list of all active servers"
        )
        self.test_get_active_server_list(server)

        print(
            f"\n\n==========================================>Activate {self.good_server_2} if down"
        )
        self.test_activate_server_if_down()

        print(
            "\n\n==========================================>Get the list of all active servers"
        )
        self.test_get_active_server_list(server)

        print("\n\n==========================================>Activate known servers")
        self.test_activate_servers_on_platform()

        print(
            "\n\n==========================================>All Tests Complete<======================================"
        )


if __name__ == "__main__":
    pass



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.


A object_action line interface for Egeria Users - all md_commands

This is an emerging capability based on the **click** package. Feedback welcome!

"""
import os
import sys

import click
from trogon import tui
from loguru import logger

from pyegeria import config_logging,  settings

from commands.cat.my_reports import start_exp2
from commands.cat.run_report import list_generic
from commands.cat.get_asset_graph import asset_viewer
from commands.cat.get_collection_tree import collection_viewer
from commands.cat.get_project_dependencies import project_dependency_viewer
from commands.cat.get_project_structure import project_structure_viewer
from commands.cat.get_tech_type_elements import tech_viewer
from commands.cat.glossary_actions import (
    create_glossary,
    create_term,
    delete_glossary,
    delete_term,
    export_terms_csv,
    import_terms_csv,
    update_term,
    update_term_status,
)

from commands.cat.collection_actions import (
    create_collection,
    delete_collection,
    update_collection,
    add_element_to_collection,
    remove_element_from_collection
)

from commands.cat.list_assets import display_assets
from commands.cat.list_cert_types import display_certifications
from commands.cat.list_collections import display_collections
from commands.cat.list_deployed_catalogs import list_deployed_catalogs
from commands.cat.list_deployed_database_schemas import (
    list_deployed_database_schemas,
)
from commands.cat.list_deployed_databases import list_deployed_databases
from commands.cat.list_glossaries import display_glossaries
from commands.cat.list_projects import display_project_list
from commands.cat.list_deployed_servers import display_servers_by_dep_imp
from commands.cat.list_tech_type_elements import list_tech_elements
from commands.cat.list_tech_types import display_tech_types
from commands.cat.list_terms import display_glossary_terms
from commands.cat.list_todos import display_to_dos as list_todos
from commands.cat.list_user_ids import list_user_ids

from commands.cat.dr_egeria_md import process_markdown_file

from commands.cli.ops_config import Config
from commands.my.list_my_profile import display_my_profile
from commands.my.list_my_roles import display_my_roles
from commands.my.monitor_my_todos import display_my_todos
from commands.my.monitor_open_todos import display_todos
from commands.my.todo_actions import (
    change_todo_status,
    create_todo,
    delete_todo,
    mark_todo_complete,
    reassign_todo,
)
from commands.ops.gov_server_actions import (
    add_catalog_target,
    refresh_gov_eng_config,
    remove_catalog_target,
    start_server,
    stop_server,
    update_catalog_target,
)
from commands.ops.list_archives import display_archive_list
from commands.ops.list_catalog_targets import display_catalog_targets
from commands.ops.load_archive import load_archive
from commands.ops.monitor_active_engine_activity import display_engine_activity
from commands.ops.monitor_engine_activity import display_engine_activity_c
from commands.ops.monitor_engine_status import display_gov_eng_status
from commands.ops.monitor_daemon_status import (
    display_integration_daemon_status,
)
from commands.ops.monitor_platform_status import (
    display_status as p_display_status,
)
from commands.ops.monitor_server_startup import display_startup_status
from commands.ops.monitor_server_status import (
    display_status as s_display_status,
)
from commands.ops.refresh_integration_daemon import refresh_connector
from commands.ops.restart_integration_daemon import restart_connector
from commands.tech.get_element_info import display_elements
from commands.tech.get_guid_info import display_guid
from commands.tech.get_tech_details import tech_details_viewer
from commands.tech.get_tech_type_template import template_viewer
from commands.tech.list_all_om_type_elements import list_elements
from commands.tech.list_all_om_type_elements_x import list_elements_x
from commands.tech.list_all_related_elements import list_related_elements
from commands.tech.list_anchored_elements import display_anchored_elements
from commands.tech.list_asset_types import display_asset_types
from commands.tech.list_elements_by_classification_by_property_value import (
    find_elements_by_classification_by_prop_value,
)
from commands.tech.list_elements_by_property_value import (
    find_elements_by_prop_value,
)
from commands.tech.list_elements_by_property_value_x import (
    find_elements_by_prop_value_x,
)
from commands.tech.list_elements_for_classification import (
    list_classified_elements,
)
from commands.tech.list_gov_action_processes import display_gov_processes
from commands.tech.list_information_supply_chains import supply_chain_viewer
from commands.tech.list_registered_services import display_registered_svcs
from commands.tech.list_related_elements_with_prop_value import (
    list_related_elements_with_prop_value,
)
from commands.tech.list_related_specification import (
    display_related_specification,
)
from commands.tech.list_relationship_types import display_relationship_types
from commands.tech.list_relationships import list_relationships
from commands.tech.list_solution_blueprints import blueprint_list
from commands.tech.list_solution_components import solution_component_list
from commands.tech.list_solution_roles import solution_role_list
from commands.tech.list_tech_templates import display_templates_spec
from commands.tech.list_valid_metadata_values import display_metadata_values
from commands.tech.generic_actions import delete_element

import sys

# Enable remote debugging (set to True when debugging)
ENABLE_DEBUG = False
#
# if ENABLE_DEBUG:
#     try:
#         import pydevd_pycharm
#
#         pydevd_pycharm.settrace('localhost', port=5678, stdout_to_server=True, stderr_to_server=True)
#     except ImportError:
#         print("pydevd-pycharm not installed. Install with: pip install pydevd-pycharm")
#     except Exception as e:
#         print(f"Could not connect to debugger: {e}")


EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")
app_settings = settings
app_config = app_settings.Environment
# config_logging()

@tui()
# @tui('menu', 'menu', 'A textual object_action line interface')
@click.version_option("5.4 ", prog_name="hey_egeria")
@click.group()
@click.option(
    "--server",
    default=app_config.egeria_metadata_store,
    help="Egeria metadata store to work with",
)
@click.option(
    "--url",
    default=app_config.egeria_platform_url,
    help="URL of Egeria metadata store platform to connect to",
)
@click.option(
    "--integration_daemon",
    default=app_config.egeria_integration_daemon,
    help="Egeria integration daemon to work with",
)
@click.option(
    "--integration_daemon_url",
    default=app_config.egeria_integration_daemon_url,
    help="URL of Egeria integration daemon platform to connect to",
)
@click.option(
    "--view_server",
    default=app_config.egeria_view_server,
    help="Egeria view server to work with",
)
@click.option(
    "--view_server_url",
    default=app_config.egeria_view_server_url,
    help="URL of Egeria view server platform to connect to",
)
@click.option(
    "--engine_host",
    default=app_config.egeria_engine_host,
    help="Egeria engine host to work with",
)
@click.option(
    "--engine_host_url",
    default=app_config.egeria_engine_host_url,
    help="URL of Egeria engine host platform to connect to",
)

@click.option(
    "--userid",
    default=EGERIA_USER,
    help="Egeria user",
)
@click.option(
    "--password",
    default=EGERIA_USER_PASSWORD,
    help="Egeria user password",
)
@click.option("--timeout", default=60, help="Number of seconds to wait")
@click.option(
    "--jupyter",
    is_flag=True,
    type=bool,
    default=app_config.egeria_jupyter,
    help="Enable for rendering in a Jupyter terminal",
)
@click.option(
    "--width",
    default=app_config.console_width,
    type=int,
    help="Screen width, in characters, to use",
)
@click.option(
    "--home_glossary_name",
    default=app_settings.User_Profile.egeria_home_glossary_name,
    help="Glossary name to use as the home glossary",
)
@click.option(
    "--glossary_path",
    default=app_config.egeria_glossary_path,
    help="Path to glossary import/export files",
)

@click.option(
    "--root_path",
    default=app_config.pyegeria_root,
    help="Root path to use for file operations",
)

@click.option(
    "--inbox_path",
    default=app_config.dr_egeria_inbox,
    help="Path to inbox files",
)
@click.option(
    "--outbox_path",
    default=app_config.dr_egeria_outbox,
    help="Path to outbox files",
)

@click.pass_context
def cli(
    ctx,
    server,
    url,
    view_server,
    view_server_url,
    integration_daemon,
    integration_daemon_url,
    engine_host,
    engine_host_url,
    userid,
    password,
    timeout,
    jupyter,
    width,
    home_glossary_name,
    glossary_path,
    root_path,
    inbox_path,
    outbox_path,
):
    """An Egeria Command Line interface for Operations"""
    ctx.obj = Config(
        server,
        url,
        view_server,
        view_server_url,
        integration_daemon,
        integration_daemon_url,
        engine_host,
        engine_host_url,
        userid,
        password,
        timeout,
        jupyter,
        width,
        home_glossary_name,
        glossary_path,
        root_path,
        inbox_path,
        outbox_path
    )
    ctx.max_content_width = 300
    ctx.ensure_object(Config)


#
#  my: Show
#
@cli.group("my")
@click.pass_context
def my(ctx):
    "Work with my information"
    pass


@my.group("show")
@click.pass_context
def my_show(ctx):
    """Display an Egeria Object"""
    pass


@my_show.command("my-reports")
@click.pass_context
def show_my_reports(ctx):
    """Display my profiles

    Usage: show my-profile

    """
    c = ctx.obj
    start_exp2(
        # c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width
    )

@my_show.command("my-profile")
@click.pass_context
def show_my_profile(ctx):
    """Display my profiles

    Usage: show my-profile

    """
    c = ctx.obj
    display_my_profile(
        c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width
    )


@my_show.command("my-roles")
@click.pass_context
def show_my_roles(ctx):
    """Display my roles"""
    c = ctx.obj
    display_my_roles(
        c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width
    )


@my_show.command("my-to-dos")
@click.pass_context
def show_my_todos(ctx):
    """Show my To-Dos

    Usage: show my-to-dos

    """
    c = ctx.obj
    display_my_todos(
        c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width
    )


@my_show.command("open-to-dos")
@click.pass_context
def show_open_todos(ctx):
    """Display Open Todo items"""
    c = ctx.obj
    display_todos(
        c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width
    )


#
#  my: Tell
#


@my.group("tell")
@click.pass_context
def my_tell(ctx):
    """Perform actions an Egeria Objects"""
    pass


my_tell.add_command(create_todo)
my_tell.add_command(delete_todo)
my_tell.add_command(change_todo_status)
my_tell.add_command(mark_todo_complete)
my_tell.add_command(reassign_todo)



#
#  tech User: Show
#
@cli.group("tech")
@click.pass_context
def tech(ctx):
    """Commands for tech Users"""
    pass

@tech.group("tell")
@click.pass_context
def tech_tell(ctx):
    """Perform actions on Egeria Objects"""
    pass


tech_tell.add_command(delete_element)

@tech.group("show")
@click.pass_context
def show_tech(ctx):
    """Display an Egeria Object"""
    pass


@show_tech.group("tech-info")
@click.pass_context
def show_tech_info(ctx):
    """Show various Egeria information"""
    pass


@show_tech.group("tech-types")
@click.pass_context
def show_tech_type(ctx):
    """Show information about Egeria technology types"""
    pass


@show_tech.group("elements")
@click.pass_context
def show_elements(ctx):
    """Show information about Egeria elements"""


@show_tech.group("supply-chains")
@click.pass_context
def show_supply_chains(ctx):
    """Show information about Information Supply Chainss"""


@show_elements.command("guid-info")
@click.argument("guid", nargs=1)
@click.pass_context
def show_guid_info(ctx, guid):
    """Display guid information

    Usage: show guid-info <a guid>

    """
    c = ctx.obj
    display_guid(guid, c.server, c.url, c.userid, c.password, c.jupyter, c.width)


@show_elements.command("anchored_elements")
@click.pass_context
@click.option(
    "--property_value",
    default="DeployedDatabaseSchema",
    help="value we are searching for",
)
@click.option(
    "--prop-list",
    default="anchorTypeName",
    help="List of properties we are searching",
)
def list_anchored_elements(ctx, property_value: str, prop_list: str):
    """List anchored elements with the specified properties"""
    c = ctx.obj
    if type(prop_list) is str:
        property_names = prop_list.split(",")
    elif type(prop_list) is list:
        property_names = prop_list
    else:
        property_names = []
        print(f"\nError --> Invalid property list - must be a string or list")
        sys.exit(4)
    display_anchored_elements(
        property_value,
        [property_names],
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.timeout,
        c.jupyter,
        c.width,
    )


@show_elements.command("elements-by-classification")
@click.option(
    "--om-type",
    default="Referenceable",
    help="Open Metadata type to filter by",
)
@click.option(
    "--classification",
    default="GovernanceProject",
    help="Classification to filter byt",
)
@click.pass_context
def show_elements_by_classification(ctx, om_type, classification):
    """Show elements by classification"""
    c = ctx.obj
    list_classified_elements(
        om_type,
        classification,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@show_elements.command("elements-by-classification-by-prop-value")
@click.option(
    "--classification",
    default="GovernanceProject",
    help="Classification to filter by",
)
@click.option(
    "--property_value",
    help="Property value to filter by",
)
@click.option(
    "--property_names",
    help="List of properties to search by",
)
@click.option(
    "--om-type",
    default="Referenceable",
    help="Open Metadata type to filter by",
)
@click.pass_context
def show_elements_by_classification_by_prop(
    ctx, classification, property_value, property_names, om_type
):
    """Show elements by classification and property value"""
    c = ctx.obj
    find_elements_by_classification_by_prop_value(
        om_type,
        classification,
        property_value,
        [property_names],
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@show_elements.command("elements-by-prop-value")
@click.option(
    "--property_value",
    help="Property value to filter by",
)
@click.option(
    "--property_names",
    help="List of properties to search by",
)
@click.option(
    "--om-type",
    default="Referenceable",
    help="Open Metadata type to filter by",
)
@click.option(
    "--extended",
    is_flag=True,
    default=False,
    help="If True, feedback information is displayed",
)
@click.pass_context
def show_elements_by_classification_by_prop(
    ctx, property_value, property_names, om_type, extended
):
    """Show elements by classification and property value"""
    c = ctx.obj
    if extended:
        find_elements_by_prop_value_x(
            om_type,
            property_value,
            [property_names],
            c.view_server,
            c.view_server_url,
            c.userid,
            c.password,
            c.jupyter,
            c.width,
        )
    else:
        find_elements_by_prop_value(
            om_type,
            property_value,
            [property_names],
            c.view_server,
            c.view_server_url,
            c.userid,
            c.password,
            c.jupyter,
            c.width,
        )


@show_elements.command("related-elements")
@click.option(
    "--element-guid",
    help="GUID of the Element to navigate from.",
)
@click.option(
    "--om-type",
    default="Referenceable",
    help="Open metadata type to filter by.",
)
@click.option(
    "--rel-type",
    default="Certification",
    help="Relationship type to follow.",
)
@click.pass_context
def show_related_elements(ctx, element_guid, om_type, rel_type):
    """Show all elements related to specified guid"""
    c = ctx.obj
    list_related_elements(
        element_guid,
        om_type,
        rel_type,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@show_elements.command("related-elements_by_prop")
@click.option(
    "--element-guid",
    help="GUID of the Element to navigate from.",
)
@click.option(
    "--rel-type",
    default="Certification",
    help="Relationship type to follow.",
)
@click.option(
    "--property_value",
    help="Property value to filter by",
)
@click.option(
    "--property_names",
    help="List of properties to search by",
)
@click.option(
    "--om-type",
    default="Referenceable",
    help="Open metadata type to filter by.",
)
@click.pass_context
def show_related_elements(
    ctx, element_guid, rel_type, property_value, property_names, om_type
):
    """Show elements related to specified guid and property value"""
    c = ctx.obj
    list_related_elements_with_prop_value(
        element_guid,
        rel_type,
        property_value,
        [property_names],
        om_type,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@show_elements.command("related-specifications")
@click.pass_context
@click.argument("element-guid")
def show_related_specifications(ctx, element_guid):
    """List specifications related to the given Element"""
    c = ctx.obj
    display_related_specification(
        element_guid,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@show_tech_type.command("list")
@click.option("--search-string", default="*", help="Tech type to search for")
@click.pass_context
def show_tech_types(ctx, search_string):
    """List deployed technology types

    Usage: show tech-types <optional search-string>

    All tech-types will be returned if no search-string is specified.

    """

    c = ctx.obj
    display_tech_types(search_string, c.view_server, c.view_server_url, c.userid, c.password)


@show_tech_type.command("details")
@click.argument("tech-name")
@click.pass_context
def show_tech_details(ctx, tech_name):
    """Show technology type details"""
    c = ctx.obj
    tech_details_viewer(tech_name, c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width)


@show_tech_type.command("templates")
@click.option(
    "--tech-type",
    default="PostgreSQL Server",
    help="Specific tech type to get elements for",
)
@click.pass_context
def show_tech_type_templates(ctx, tech_type):
    """List technology type templates"""
    c = ctx.obj
    template_viewer(
        tech_type,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@show_tech_info.command("asset-types")
@click.pass_context
def show_asset_types(ctx):
    """Display asset types"""
    c = ctx.obj
    display_asset_types(
        c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width
    )


@show_tech_info.command("registered-services")
@click.option(
    "--services",
    type=click.Choice(
        [
            "all",
            "access-services",
            "common-services",
            "engine-services",
            "governance-services",
            "integration-services",
            "view-services",
        ],
        case_sensitive=False,
    ),
    default="all",
    help="Which service group to display",
)
@click.pass_context
def show_registered_services(ctx, services):
    """Show information about a registered services"""
    c = ctx.obj
    display_registered_svcs(
        services,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@show_tech_info.command("relationship-types")
@click.option(
    "--om-type",
    default="Referenceable",
    help="Relationship type to get information about",
)
@click.pass_context
def show_relationship_types(ctx, om_type):
    """Show information about the specified relationship types"""
    c = ctx.obj
    display_relationship_types(
        om_type,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        False,
        c.jupyter,
        c.width,
    )


@show_tech_type.command("template-spec")
@click.pass_context
@click.option(
    "--search-string", default="*", help="Technology type to get information about"
)
def tech_template_spec(ctx, search_string):
    """Display template specification information about the specified technology."""
    c = ctx.obj
    display_templates_spec(
        search_string,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@show_tech_info.command("gov-action-processes")
@click.pass_context
@click.option("--search-string", default="*", help="Search string")
def gov_action_processes(ctx, search_string):
    """Display available governance action processes."""
    c = ctx.obj
    display_gov_processes(
        search_string,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@show_tech_info.command("relationships")
@click.option(
    "--relationship",
    default="Certification",
    help="Relationship type name to search for.",
)
@click.pass_context
def show_relationships(ctx, relationship):
    """Show the structure of the project starting from a root project"""
    c = ctx.obj
    list_relationships(
        relationship,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.timeout,
        c.jupyter,
        c.width,
    )


@show_tech_info.command("valid-metadata-values")
@click.pass_context
@click.option("--property", default="projectHealth", help="Metadata property to query")
@click.option("--type-name", default="Project", help="Metadata type to query")
def valid_metadata_values(ctx, property, type_name):
    """Display the valid metadata values for a property and type"""
    c = ctx.obj
    display_metadata_values(
        property,
        type_name,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        False,
        c.jupyter,
        c.width,
    )


@show_elements.command("elements")
@click.pass_context
@click.option(
    "--extended",
    is_flag=True,
    default=False,
    help="If True, feedback information is displayed",
)
@click.option("--om_type", default="Referenceable", help="Metadata type to query")
def list_all_om_type_elements(ctx, om_type, extended):
    """Display all elements of a specific Open Metadata Type"""
    c = ctx.obj
    if extended:
        list_elements_x(
            om_type,
            c.view_server,
            c.view_server_url,
            c.userid,
            c.password,
            c.jupyter,
            c.width,
        )
    else:
        list_elements(
            om_type,
            c.view_server,
            c.view_server_url,
            c.userid,
            c.password,
            c.jupyter,
            c.width,
        )


@show_tech_info.command("processes")
@click.pass_context
def list_element_info(ctx):
    """Display the governance action processes"""
    c = ctx.obj
    list_elements(
        "GovernanceActionProcess",
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@show_elements.command("get-elements")
@click.pass_context
@click.option("--om_type", default="Referenceable", help="Metadata type to query")
def get_element_info(ctx, om_type):
    """Display a table of elements of an Open Metadata Type"""
    c = ctx.obj
    display_elements(
        om_type,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@show_supply_chains.command("supply-chains")
@click.option("--search-string", default="*", help="Search string")
@click.pass_context
def list_supply_chains(ctx, search_string):
    """Display supply chains"""
    c = ctx.obj
    supply_chain_viewer(
        search_string,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@show_supply_chains.command("blueprints")
@click.option("--search-string", default="*", help="Search string")
@click.pass_context
def list_blueprints(ctx, search_string):
    """Display solution blueprints"""
    c = ctx.obj
    blueprint_list(search_string, c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width)


@show_supply_chains.command("solution-roles")
@click.option("--search-string", default="*", help="Search string")
@click.pass_context
def list_solution_roles(ctx, search_string):
    """Display solution roles"""
    c = ctx.obj
    solution_role_list(
        search_string,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@show_supply_chains.command("solution-components")
@click.option("--search-string", default="*", help="Search string")
@click.pass_context
def list_solution_components(ctx, search_string):
    """Display solution componentss"""
    c = ctx.obj
    solution_component_list(
        search_string,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


#
#   Catalog User: Show
#


@cli.group("cat")
@click.pass_context
def cat(ctx):
    """Commands for all users"""
    pass


@cat.group("show")
@click.pass_context
def show_cat(ctx):
    """Display an Egeria Object"""
    pass

@show_cat.group("data_designer")
@click.pass_context
def show_cat_design(ctx):
    """Group of commands for Egeria Data Designer"""
    pass


@show_cat_design.command("data-fields")
@click.option("--output-format", default = "TABLE", help="Output format type")
@click.option('--search-string', default="*", help="Search string")
@click.pass_context
def show_data_fields(ctx, output_format, search_string):
    """Report on Data Fields"""
    c = ctx.obj
    logger.info(f"Hey Egeria: view server @ {c.view_server_url}")
    list_generic(
        "Data-Fields", output_format = output_format, view_server = c.view_server, view_url= c.view_server_url,
        user = c.userid, user_pass = c.password, params = {"search_string": search_string}, render_table = True
    )


@show_cat_design.command("data-structures")
@click.option("--output-format", default = "TABLE", help="Output format type")
@click.option('--search-string', default="*", help="Search string")
@click.pass_context
def show_data_structures(ctx, output_format, search_string):
    """Reporton Data Structures"""
    c = ctx.obj
    logger.info(f"Hey Egeria: view server @ {c.view_server_url}")
    list_generic(
        "Data-Structures", output_format = output_format, view_server = c.view_server, view_url= c.view_server_url,
        user = c.userid, user_pass = c.password, params = {"search_string": search_string}, render_table = True
    )

@show_cat_design.command("data-classes")
@click.option("--output-format", default = "TABLE", help="Output format type")
@click.option('--search-string', default="*", help="Search string")
@click.pass_context
def show_data_classes(ctx, output_format, search_string):
    """Dynamically generate output based on a format set"""
    c = ctx.obj
    logger.info(f"Hey Egeria: view server @ {c.view_server_url}")
    list_generic(
        "Data-Classes", output_format = output_format, view_server = c.view_server, view_url= c.view_server_url,
        user = c.userid, user_pass = c.password, params = {"search_string": search_string}, render_table = True
    )

@show_cat_design.command("data-dictionaries")
@click.option("--output-format", default = "TABLE", help="Output format type")
@click.option('--search-string', default="*", help="Search string")
@click.pass_context
def show_data_dictionaries(ctx, output_format, search_string):
    """Report on Data Dictionaries"""
    c = ctx.obj
    logger.info(f"Hey Egeria: view server @ {c.view_server_url}")
    list_generic(
        "Data-Dictionaries", output_format = output_format, view_server = c.view_server, view_url= c.view_server_url,
        user = c.userid, user_pass = c.password, params = {"search_string": search_string}, render_table = True
    )

@show_cat_design.command("data-specifications")
@click.option("--output-format", default = "TABLE", help="Output format type")
@click.option('--search-string', default="*", help="Search string")
@click.pass_context
def show_data_specs(ctx, output_format, search_string):
    """Report on Data Specifications"""
    c = ctx.obj
    logger.info(f"Hey Egeria: view server @ {c.view_server_url}")
    list_generic(
        "Data-Specifications", output_format = output_format, view_server = c.view_server, view_url= c.view_server_url,
        user = c.userid, user_pass = c.password, params = {"search_string": search_string}, render_table = True
    )




@show_cat.group("info")
@click.pass_context
def show_cat_info(ctx):
    """Group of md_commands to show information about various Egeria objects"""
    pass

@show_cat_info.command("Run Report")
@click.option("--report", help="Report to output", default="Digital-Products")
@click.option("--output-format", default = "TABLE", help="Output format type")
@click.option('--search-string', default="*", help="Search string")
@click.pass_context
def show_format_set(ctx, report, output_format, search_string):
    """Dynamically generate output based on a format set"""
    c = ctx.obj
    logger.info(f"Hey Egeria: view server @ {c.view_server_url}")
    list_generic(
        report, output_format = output_format, view_server = c.view_server, view_url = c.view_server_url,
        user = c.userid, user_pass = c.password, params = {"search_string": search_string},render_table = True
    )


@show_cat_info.command("tech-types")
@click.option("--tech_type", default="*", help="Tech type to search for")
@click.pass_context
def show_tech_types(ctx, tech_type):
    """List deployed technology types"""
    c = ctx.obj
    display_tech_types(tech_type, c.view_server, c.view_server_url, c.userid, c.password)


@show_cat_info.command("collections")
@click.option("--collection", default="*", help="Collection to search for")
@click.pass_context
def show_collections(ctx, collection):
    """List Collections"""
    c = ctx.obj
    display_collections(
        collection, c.view_server, c.view_server_url, c.userid, c.password
    )


@show_cat.group("assets")
@click.pass_context
def asset_group(ctx):
    """Show assets known to Egerias"""
    pass


@asset_group.command("elements-of-tech-type")
@click.option(
    "--tech_type",
    default="PostgreSQL Server",
    help="Specific tech type to get elements for",
)
@click.pass_context
def show_tech_type_elements(ctx, tech_type):
    """List technology type elements"""
    c = ctx.obj
    tech_viewer(tech_type, c.view_server, c.view_server_url, c.userid, c.password)


@asset_group.command("in-asset-domain")
@click.argument("search-string")
@click.pass_context
def show_assets(ctx, search_string):
    """Find and display assets in domain

    Usage: show assets <search-string>

           search-string must be greater than four characters.
    """
    c = ctx.obj
    display_assets(
        search_string,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        60,
        c.jupyter,
        c.width,
    )


@asset_group.command("asset-graph")
@click.argument("asset_guid", nargs=1)
@click.pass_context
def show_asset_graph(ctx, asset_guid):
    """Display a tree graph of information about an asset

    Usage: show asset-graph <asset-guid>

           asset-guid must be a valid asset guid. These can be found through other md_commands such as 'show tech-type-elements'

    """
    c = ctx.obj
    asset_viewer(
        asset_guid,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@show_cat.group("glossary")
@click.pass_context
def glossary_group(ctx):
    """Show glossary information"""
    pass


@glossary_group.command("glossary-terms")
@click.option(
    "--search-string",
    default="*",
    help="List glossary terms similar to search string - minimum of 4 characters",
)
@click.option(
    "--glossary-guid",
    default=os.environ.get("EGERIA_HOME_GLOSSARY_GUID", None),
    help="Optionally restrict search to glossary with the specified guid",
)
@click.option(
    "--glossary-name",
    default="*",
    help="Optionally restrict search to a specific named glossary",
)
@click.option(
    "--output-format",
    type=click.Choice(["FORM", "REPORT", "TABLE"]),
    default="TABLE",
    help="Display on screen as table, or as FORM or REPORT file",
    )
@click.pass_context
def show_terms(ctx, search_string, glossary_guid, glossary_name, output_format):
    """Find and display glossary terms"""
    c = ctx.obj
    display_glossary_terms(search_string, glossary_guid, glossary_name, c.view_server, c.view_server_url, c.userid,
                           c.password, c.jupyter, c.width, output_format)



@glossary_group.command("glossaries")
@click.option("--search_string", default="*", help="Name to search for glossaries")
@click.option(
    "--output-format",
    type=click.Choice(["FORM", "REPORT", "TABLE"]),
    default="TABLE",
    help="Display on screen as table, or as FORM or REPORT file",
    )
@click.pass_context
def glossaries(ctx, search_string, output_format):
    """Display a list of glossaries"""
    c = ctx.obj
    display_glossaries(
        search_string,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
        output_format,
    )


@show_cat_info.command("collection-graph")
@click.option(
    "--root_collection",
    default="Coco Pharmaceuticals Governance Domains",
    help="View of tree of collections from a given root",
)
@click.pass_context
def show_collection(ctx, root_collection):
    """Display collection graph"""
    c = ctx.obj
    collection_viewer(
        root_collection,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@show_cat.group("projects")
@click.pass_context
def show_project_group(ctx):
    """Show project information in Egeria"""
    pass


@show_project_group.command("projects")
@click.option("--search-string", default="*", help="List Projects by Search String")
@click.pass_context
def show_projects(ctx, search_string):
    """Display a list of Egeria projects"""
    c = ctx.obj
    display_project_list(
        search_string,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        False,
        c.jupyter,
        c.width,
    )


@show_cat_info.command("certification-types")
@click.option("--search-string", default="CertificationType", help="")
@click.pass_context
def show_certification_types(ctx, search_string):
    """Show certification types"""
    c = ctx.obj
    display_certifications(search_string, c.view_server, c.view_server_url, c.userid, c.password, c.timeout, c.jupyter)


@show_cat_info.command("asset-types")
@click.pass_context
def show_asset_types(ctx):
    """Display known asset types"""
    c = ctx.obj
    display_asset_types(
        c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width
    )


@show_project_group.command("project-structure")
@click.option(
    "--project",
    default="Clinical Trials Management",
    help="Enter the root project to start from",
)
@click.pass_context
def show_project_structure(ctx, project):
    """Show the organization structure of the project starting from a root project"""
    c = ctx.obj
    project_structure_viewer(
        project,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
        c.timeout,
    )


@show_project_group.command("project-dependencies")
@click.option(
    "--project",
    default="Clinical Trials Management",
    help="Enter the root project to start from",
)
@click.pass_context
def show_project_dependencies(ctx, project):
    """Show the dependencies of a project starting from a root project"""
    c = ctx.obj
    project_dependency_viewer(
        project,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
        c.timeout,
    )


@show_cat_info.command("to-dos")
@click.option("--search-string", default="*", help="View the list of To-Do items")
@click.option(
    "--status",
    type=click.Choice(
        ["OPEN", "IN_PROGRESS", "WAITING", "COMPLETE", "ABANDONED", "None"],
        case_sensitive=False,
    ),
    help="Enter an optional status filter",
    required=False,
    default="NONE",
)
@click.pass_context
def show_todos(ctx, search_string, status):
    """Display list of To Dost"""
    c = ctx.obj
    list_todos(
        search_string,
        status,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@show_cat_info.command("user-ids")
@click.pass_context
def show_todos(ctx):
    """Display a list of known user-ids"""
    c = ctx.obj
    list_user_ids(
        c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width
    )


@show_cat.group("deployed-data")
@click.pass_context
def deployed_data(ctx):
    """Show deployed data resources known to Egeria"""
    pass


@deployed_data.command("deployed-servers")
@click.option(
    "--search-string",
    default="*",
    help="Filter deployed for deployed implementation type by search string",
)
@click.pass_context
def show_deployed_servers(ctx, search_string):
    """Display deployed servers"""
    c = ctx.obj
    display_servers_by_dep_imp(search_string, c.view_server, c.view_server_url, c.userid, c.password, c.jupyter,
                               c.width)


@deployed_data.command("deployed-schemas")
@click.option("--catalog", default="*", help="What database or catalog to search")
@click.pass_context
def deployed_schemas(ctx, catalog):
    """Display deployed schemas"""
    c = ctx.obj
    list_deployed_database_schemas(
        catalog,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@deployed_data.command("catalogs")
@click.option("--search_server", default="*", help="Server to search for catalogs")
@click.pass_context
def list_catalogs(ctx, search_server):
    """Display deployed catalogs"""
    c = ctx.obj
    list_deployed_catalogs(
        search_server,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@deployed_data.command("databases")
@click.pass_context
def list_databases(ctx):
    """Display deployed databases"""
    c = ctx.obj
    list_deployed_databases(c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width)


#
#  Tell-cat
#


@cat.group("tell")
@click.pass_context
def tell_cat(ctx):
    """Perform actions an Egeria Objects"""
    pass

#
# dr.egeria
#
@tell_cat.group("dr_egeria")
@click.pass_context
def dr_egeria(ctx):
    """Execute Dr.Egeria actions"""
    pass

dr_egeria.add_command(process_markdown_file)



@tell_cat.group("glossary")
@click.pass_context
def tell_glossary(ctx):
    """Perform glossary actions"""
    pass


tell_glossary.add_command(create_glossary)
tell_glossary.add_command(delete_glossary)
tell_glossary.add_command(delete_term)
tell_glossary.add_command(create_term)
tell_glossary.add_command(import_terms_csv)
tell_glossary.add_command(export_terms_csv)
tell_glossary.add_command(update_term)
tell_glossary.add_command(update_term_status )

@tell_cat.group("collection")
@click.pass_context
def tell_collections(ctx):
    """Perform actions on collections"""
    pass

tell_collections.add_command(create_collection)
tell_collections.add_command(delete_collection)
tell_collections.add_command(update_collection)
tell_collections.add_command(add_element_to_collection)
tell_collections.add_command(remove_element_from_collection)


@tell_cat.group("todo")
@click.pass_context
def tell_cat_todo(ctx):
    """Perform todo actions"""
    pass


tell_cat_todo.add_command(mark_todo_complete)
tell_cat_todo.add_command(reassign_todo)
tell_cat_todo.add_command(delete_todo)
tell_cat_todo.add_command(create_todo)


@show_cat_info.command("tech-types")
@click.option("--tech_type", default="*", help="Tech type to search for")
@click.pass_context
def show_tech_types(ctx, tech_type):
    """List deployed technology types"""
    c = ctx.obj
    display_tech_types(tech_type, c.view_server, c.view_server_url, c.userid, c.password)


@show_cat_info.command("certification-types")
@click.option("--search-string", default="CertificationType", help="")
@click.pass_context
def show_certification_types(ctx, search_string):
    """Show certification types"""
    c = ctx.obj
    display_certifications(search_string, c.view_server, c.view_server_url, c.userid, c.password, c.timeout, c.jupyter)


@show_project_group.command("project-structure")
@click.option(
    "--project",
    default="Clinical Trials Management",
    help="Enter the root project to start from",
)
@click.pass_context
def show_project_structure(ctx, project):
    """Show the organization structure of the project starting from a root project"""
    c = ctx.obj
    project_structure_viewer(
        project,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
        c.timeout,
    )


@show_project_group.command("project-dependencies")
@click.option(
    "--project",
    default="Clinical Trials Management",
    help="Enter the root project to start from",
)
@click.pass_context
def show_project_dependencies(ctx, project):
    """Show the dependencies of a project starting from a root project"""
    c = ctx.obj
    project_dependency_viewer(
        project,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
        c.timeout,
    )



@asset_group.command("asset-graph")
@click.argument("asset_guid", nargs=1)
@click.pass_context
def show_asset_graph(ctx, asset_guid):
    """Display a tree graph of information about an asset

    Usage: show asset-graph <asset-guid>

           asset-guid must be a valid asset guid. These can be found through other md_commands such as 'show tech-type-elements'

    """
    c = ctx.obj
    asset_viewer(
        asset_guid,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@asset_group.command("tech-type-elements")
@click.option(
    "--tech_type",
    default="PostgreSQL Server",
    help="Specific tech type to get elements for",
)
@click.pass_context
def list_tech_type_elements(ctx, tech_type):
    """List technology type elements"""
    c = ctx.obj
    list_tech_elements(
        tech_type, c.view_server, c.view_server_url, c.userid, c.password
    )


@show_project_group.command("projects")
@click.option("--search-string", default="*", help="List Projects by Search String")
@click.pass_context
def show_projects(ctx, search_string):
    """Display a list of Egeria projects"""
    c = ctx.obj
    display_project_list(
        search_string,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        False,
        c.jupyter,
        c.width,
    )


@show_cat_info.command("to-dos")
@click.option("--search-string", default="*", help="View the list of To-Do items")
@click.option(
    "--status",
    type=click.Choice(
        ["OPEN", "IN_PROGRESS", "WAITING", "COMPLETE", "ABANDONED", "None"],
        case_sensitive=False,
    ),
    help="Enter an optional status filter, default='OPEN'",
    required=False,
    default="OPEN",
)
@click.pass_context
def show_todos(ctx, search_string, status):
    """Display a list of To Dos"""
    c = ctx.obj
    list_todos(
        search_string,
        status,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@show_cat_info.command("user-ids")
@click.pass_context
def show_user_ids(ctx):
    """Display table of known user ids"""
    c = ctx.obj
    list_user_ids(
        c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width
    )


@deployed_data.command("deployed-servers")
@click.option(
    "--search-string",
    default="*",
    help="Filter deployed for deployed implementation type by search string",
)
@click.pass_context
def show_deployed_servers(ctx, search_string):
    """Show list of deployed servers"""
    c = ctx.obj
    display_servers_by_dep_imp(search_string, c.view_server, c.view_server_url, c.userid, c.password, c.jupyter,
                               c.width)


@deployed_data.command("deployed-schemas")
@click.option(
    "--search_catalog", default="*", help="What database or catalog to search"
)
@click.pass_context
def deployed_schemas(ctx, search_catalog):
    """Display a list of deployed schemas"""
    c = ctx.obj
    list_deployed_database_schemas(
        search_catalog,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@deployed_data.command("catalogs")
@click.option("--search_server", default="*", help="Server to search for catalogs")
@click.pass_context
def catalogs(ctx, search_server):
    """Display a list of deployed catalogs"""
    c = ctx.obj
    list_deployed_catalogs(
        search_server,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@deployed_data.command("databases")
@click.pass_context
def databases(ctx):
    """Display a list of deployed databases"""
    c = ctx.obj
    list_deployed_databases(c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width)


# @tell_cat.group("survey")
# @click.pass_context
# def survey(ctx):
#     """Refresh the specified integration connector or ALL connectors if not specified"""
#     c = ctx.obj
#     pass
#
#
# @survey.object_action("survey-uc-server")
# @click.pass_context
# @click.option(
#     "--uc_endpoint",
#     default="https://localhost:8080",
#     help="Endpoint of the Unity Catalog Server to Survey",
# )
# def survey_uc_server(ctx, uc_endpoint):
#     """Survey the Unity Catalog server at the given endpoint"""
#     c = ctx.obj
#     pass
# restart_connector(connector, c.integration_daemon, c.integration_daemon_url,
#                   c.userid, c.password)


#
#  Operations: Show
#


@cli.group("ops")
@click.pass_context
def ops(ctx):
    """Commands to understand and manage operations"""
    pass


@ops.group("show")
@click.pass_context
def show_ops(ctx):
    """Display an Egeria Object"""
    pass


@show_ops.group("repository")
@click.pass_context
def show_repo(ctx):
    """Group of md_commands to show repository information"""
    pass


@show_repo.command("archives")
@click.pass_context
def show_archives(ctx):
    c = ctx.obj
    display_archive_list(
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        False,
        c.jupyter,
        c.width,
    )


@show_ops.group("platforms")
@click.pass_context
def show_platform(ctx):
    """Group of md_commands to show information about Egeria platforms"""
    pass


@show_platform.command("status")
@click.pass_context
def show_platform_status(ctx):
    """Display a live status view of known platforms"""
    c = ctx.obj
    p_display_status(
        c.view_server, c.view_server_url, c.userid, c.password
    )


@show_ops.group("servers")
@click.pass_context
def show_server(ctx):
    """Group of md_commands to show information about Egeria servers"""
    pass


@show_server.command("status")
@click.option(
    "--full",
    is_flag=True,
    default=False,
    help="If set, full server descriptions will be shown",
)
@click.pass_context
def show_server_status(ctx, full):
    """Display a live status view of Egeria servers for the specified Egeria platform"""
    c = ctx.obj
    s_display_status(
        full,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@show_server.command("startup")
@click.pass_context
def show_startup_status(ctx):
    """Display a live status view of Egeria servers for the specified Egeria platform"""
    c = ctx.obj
    display_startup_status(
        c.server,
        c.url,
        c.userid,
        c.password,
        c.jupyter,
        c.width,
    )


@show_ops.group("engines")
@click.pass_context
def engine_host(ctx):
    """Group of md_commands to show information about Egeria engines"""
    pass


@engine_host.command("status")
@click.option(
    "--engine-list",
    default="*",
    help="Enter the list of connectors you are interested in or ['*'] for all",
)
@click.option(
    "--list", is_flag=True, default=False, help="If True, a paged list will be shown"
)
@click.pass_context
def gov_eng_status(ctx, engine_list, list):
    """Display engine-host status information"""
    c = ctx.obj
    display_gov_eng_status(
        [engine_list],
        engine_host=c.engine_host,
        view_server=c.view_server,
        url=c.view_server_url,
        username=c.userid,
        user_pass=c.password,
        paging=list,
        jupyter=c.jupyter,
        width=c.width,
    )


@engine_host.command("activity")
@click.option(
    "--rowlimit",
    default=0,
    type=int,
    show_default=True,
    help="If non-zero, limit the number of rows returned",
)
@click.option(
    "--list", is_flag=True, default=False, help="If True, a paged list will be shown"
)
@click.option(
    "--compressed",
    is_flag=True,
    default=False,
    show_default=True,
    help="Compressed combines some attributes into a single column",
)
@click.pass_context
def eng_activity_status(ctx, rowlimit: int, list: bool, compressed: bool):
    """Show Governance Activity in engine-host"""
    c = ctx.obj
    if compressed:
        display_engine_activity_c(
            rowlimit,
            c.view_server,
            c.view_server_url,
            c.userid,
            c.password,
            list,
            c.jupyter,
            c.width,
        )
    else:
        display_engine_activity(
            rowlimit,
            c.view_server,
            c.view_server_url,
            c.userid,
            c.password,
            list,
            c.jupyter,
            c.width,
        )


@show_ops.group("integrations")
@click.pass_context
def integrations(ctx):
    """Group of md_commands to show information about Egeria integrations"""
    pass


@integrations.command("status")
@click.option(
    "--connector-list",
    default="*",
    help="Enter the list of connectors you are interested in or ['*'] for all",
)
@click.option(
    "--list", is_flag=True, default=False, help="If True, a paged list will be shown"
)
@click.option(
    "--sorted", type=bool, default=True, help="If True, the table will be sorted"
)
@click.pass_context
def integrations_status(ctx, connector_list, list, sorted):
    """Display integration_daemon status information"""
    c = ctx.obj
    display_integration_daemon_status(
        [connector_list],
        c.integration_daemon,
        c.integration_daemon_url,
        c.view_server,
        c.view_server_url,
        c.userid,
        c.password,
        list,
        c.jupyter,
        c.width,
        sorted,
    )


@integrations.command("targets")
@click.pass_context
@click.argument("--connector", nargs=1)
def integrations_status(ctx, connector):
    """Display Catalog Targets for a connector"""
    c = ctx.obj
    display_catalog_targets(connector, c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width)


#
#  Operations: Tell
#


@ops.group("tell")
@click.pass_context
def tell_ops(ctx):
    """Perform actions an Egeria Objects"""
    pass


@tell_ops.group("integration-daemon")
@click.pass_context
def tell_integration_daemon(ctx):
    """Group of md_commands to an integration_daemon"""
    pass


@tell_integration_daemon.command("refresh")
@click.pass_context
@click.option(
    "--connector",
    default="all",
    help="Name of connector to refresh or 'all' to refresh all",
)
def refresh_connectors(ctx, connector):
    """Refresh the specified integration connector or ALL connectors if not specified"""
    c = ctx.obj
    refresh_connector(
        connector, c.integration_daemon, c.integration_daemon_urll, c.view_server, c.userid, c.password
    )


@tell_integration_daemon.command("restart")
@click.pass_context
@click.option(
    "--connector",
    default="all",
    help="Name of connector to restart or 'all' to restart all",
)

def restart_connectors(ctx, server, connector):
    """Restart the specified integration connector or ALL connectors if not specified"""
    c = ctx.obj
    restart_connector(
        connector, c.integration_daemon, c.integration_daemon_url, c.view_server, c.userid, c.password
    )


tell_integration_daemon.add_command(add_catalog_target)
tell_integration_daemon.add_command(remove_catalog_target)
tell_integration_daemon.add_command(update_catalog_target)
tell_integration_daemon.add_command(stop_server)
tell_integration_daemon.add_command(start_server)


@tell_ops.group("servers")
@click.pass_context
def servers(ctx):
    """Perform actions on OMAG Servers"""
    pass


servers.add_command(start_server)
servers.add_command(stop_server)


@tell_ops.group("engine-host")
@click.pass_context
def engine_host(ctx):
    """Group of md_commands to an engine-host"""
    pass


engine_host.add_command(start_server)
engine_host.add_command(stop_server)
engine_host.add_command(refresh_gov_eng_config)


@tell_ops.group("repository")
@click.pass_context
def repository(ctx):
    """Group of md_commands to a repository"""
    pass


repository.add_command(load_archive)

if __name__ == "__main__":
    while True:
        try:
            cli()
        except Exception as e:
            click.echo(f"Error: {e}")



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

A simple display for collections
"""
import argparse
import json
import os
import time

from jsonschema import ValidationError
from rich import box
from rich.console import Console
from rich.markdown import Markdown
from rich.prompt import Prompt
from rich.table import Table
from rich.text import Text
from loguru import logger
from pyegeria import (
    CollectionManager, settings,
    NO_ELEMENTS_FOUND, config_logging, load_app_config, get_app_config, init_logging, config_logging, PyegeriaException,
    print_basic_exception,PyegeriaException, )
from commands.cat.run_report import list_generic
from pyegeria._exceptions_new import print_validation_error
app_config = settings.Environment

EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")
app_settings = get_app_config(app_config.pyegeria_root+"/.env")

config_logging()

console = Console(width=app_config.console_width, force_terminal=(not app_config.egeria_jupyter))

def display_collections(
    search_string: str = "*",
    view_server: str = app_config.egeria_view_server,
    view_url: str = app_config.egeria_view_server_url,
    user: str = app_settings.User_Profile.user_name,
    user_pass: str = app_settings.User_Profile.user_pwd,
    jupyter: bool = app_config.egeria_jupyter,
    width: int = app_config.console_width,
    output_format: str = "TABLE"
):
    """Display either a specified glossary or all collections if the search_string is '*'.
    Parameters
    ----------
    search_string : str, default is '*'
        The string used to search for collections.
    view_server : str
        The view server name or address where the Egeria services are hosted.
    view_url : str
        The URL of the platform the view server is on.
    user : str
        The user ID for authentication with the Egeria server.
    user_pass : str
        The password for authentication with the Egeria server.
    jupyter : bool, optional
        A boolean indicating whether the output is intended for a Jupyter notebook (default is EGERIA_JUPYTER).
    width : int, optional
        The width of the console output (default is EGERIA_WIDTH).
    output_format : str, optional
        Format of the output. Default is TABLE.
    """

    try:
        table_caption = "Basic Collection List"
        list_generic(report_spec="BasicCollections", output_format=output_format, view_server=view_server,
                     view_url=view_url, user=user, user_pass=user_pass, params={"search_string": search_string},
                     render_table=True, write_file=True, table_caption=table_caption, use_pager=True,
                     width=width, jupyter=jupyter, prompt_missing=True)


    except (PyegeriaException) as e:
        print_basic_exception(e)
    except ValidationError as e:
        print_validation_error(e)
    except Exception as e:
        print(f"An unexpected error occurred: {e}")


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--server", help="Name of the server to display status for")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")

    args = parser.parse_args()

    server = args.server if args.server is not None else app_config.egeria_view_server
    url = args.url if args.url is not None else app_config.egeria_view_server_url
    userid = args.userid if args.userid is not None else app_settings.User_Profile.user_name
    user_pass = args.password if args.password is not None else app_settings.User_Profile.user_pwd

    try:
        search_string = Prompt.ask(
            "Enter the collection you are looking for or '*' for all:", default="*"
        ).strip()
        output_format = Prompt.ask("Which output format do you want?", choices=["DICT", "TABLE", "FORM", "REPORT", "HTML", "LIST"], default="TABLE")

        display_collections(search_string, server, url, userid, user_pass, output_format = output_format)

    except KeyboardInterrupt:
        pass

    except PyegeriaException as e:
        print_basic_exception(e)
    except ValidationError as e:
        print_validation_error(e)
    except Exception as e:
        print(f"An unexpected error occurred: {e}")


if __name__ == "__main__":
    # EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
    # EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")
    # PYEGERIA_ROOT_PATH = os.environ.get("PYEGERIA_ROOT_PATH", "/Users/dwolfson/localGit/egeria-v5-3/egeria-python")
    main()



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

A simple viewer for Information Solution Blueprints

"""

import argparse
import os
import time

from rich import box, print
from rich.console import Console
from rich.markdown import Markdown
from rich.panel import Panel
from rich.prompt import Prompt
from rich.table import Table
from rich.text import Text
from rich.tree import Tree

from pyegeria import (
    PyegeriaException,
    print_basic_exception,
    SolutionArchitect,
    settings, load_app_config, pretty_print_config,
    config_logging,
    save_mermaid_html,
)




app_config = settings.Environment
config_path = os.path.join(app_config.pyegeria_config_directory, app_config.pyegeria_config_file)
print(f"Loading config from {config_path}")
EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")
EGERIA_MERMAID_FOLDER = os.path.join(app_config.pyegeria_root, app_config.egeria_mermaid_folder)
conf = load_app_config(config_path)

print(EGERIA_MERMAID_FOLDER)
exit(0)
console = Console(width = app_config.console_width)
config_logging()

def blueprint_list(
    search_string: str,
    server_name: str,
    platform_url: str,
    user: str,
    user_password: str,
    jupyter: bool = app_config.egeria_jupyter,
    width: int = app_config.console_width,
    timeout: int = 30,
):
    """A Supply Chain viewer"""
    client = SolutionArchitect(server_name, platform_url, user, user_password)
    token = client.create_egeria_bearer_token()

    def generate_table() -> Table | str:
        table = Table(
            title=f"Blueprints matching  {search_string} @ {time.asctime()}",
            style="bright_white on black",
            header_style="bright_white on dark_blue",
            title_style="bold white on black",
            caption_style="white on black",
            show_lines=True,
            box=box.ROUNDED,
            caption=f"View Server '{server_name}' @ Platform - {platform_url}",
            expand=True,
        )
        table.add_column("Blueprint Name  /  Diagram Link", justify="center")
        table.add_column(
            "Qualified Name   /  GUID  /  Version",
            justify="center",
            width=38,
            no_wrap=False,
        )
        table.add_column("Description", justify="center")
        table.add_column("Solution Components", justify="center")

        blueprints = client.find_solution_blueprints(search_string)
        if isinstance(blueprints, list) is False:
            return "No Blueprints found"

        for bp in blueprints:
            bp_name = bp["properties"].get("displayName", "---")
            bp_qname = bp["properties"].get("qualifiedName", "---")
            bp_guid = bp["elementHeader"]["guid"]
            bp_desc = bp["properties"].get("description", "---")
            bp_unique_name = f"{bp_qname}\n\n--\n\n{bp_guid}"
            bp_mermaid = bp.get("mermaidGraph", "---")
            if bp_mermaid != "---":
                link = save_mermaid_html(
                    bp_name, bp_mermaid, f"{EGERIA_MERMAID_FOLDER}/blueprints"
                )
                link = f"file://{link}"
                bp_mermaid = Text(link, style="blue link " + link)

            bp_mermaid_label = Text(f"{bp_name}\n\n--\n\n{bp_mermaid}")

            bp_components = bp.get("solutionComponents", [])
            comp_md = ""
            first_comp = True
            for component in bp_components:
                comp = component.get("solutionComponent", "")
                if isinstance(comp, dict) is False:
                    continue
                comp_props = comp.get("properties", {})
                comp_name = comp_props.get("displayName", "---")
                comp_description = comp_props.get("description", "---")
                comp_planned = comp_props["extendedProperties"].get(
                    "plannedDeployedImplementationType", "---"
                )
                comp_type = comp_props.get("solutionComponentType", "---")

                comp_actors = comp.get("actors", [])
                comp_actors_list = f"{len(comp_actors)}\n"
                for actor in comp_actors:
                    comp_actor_role = actor["relationshipProperties"].get("role", "---")
                    comp_actor_props = actor["relatedElement"].get("properties", {})
                    comp_actor_props_md = f"\n\t* Role: {comp_actor_role}\n"
                    for prop in comp_actor_props.keys():
                        comp_actor_props_md += f"\n\t\t* {prop}: {comp_actor_props[prop]}"
                    comp_actors_list += comp_actor_props_md
                if first_comp:
                    first_comp = False
                else:
                    comp_md += "\n\n---\n\n"
                comp_md += f"Solution Component {comp_name}\n\n"
                comp_md += f"* Type: {comp_type}\n"
                comp_md += f"* Description: {comp_description}\n"
                comp_md += f"* Planned Deployment: {comp_planned}\n"
                comp_md += f"* Actors: {comp_actors_list}\n"
            comp_out = Markdown(comp_md)
            table.add_row(bp_mermaid_label, bp_unique_name, bp_desc, comp_out)

        return table

    try:
        console = Console(width=width, force_terminal=not jupyter)
        with console.pager():
            console.print(generate_table())

    except (
        PyegeriaException
    ) as e:
        print_basic_exception(e)


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument("--server", help="Name of the server to display status for")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")
    args = parser.parse_args()

    server = args.server if args.server is not None else app_config.egeria_view_server
    url = args.url if args.url is not None else app_config.egeria_platform_url
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD

    try:
        search_string = Prompt.ask("Enter a search string:", default="*")
        blueprint_list(search_string, server, url, userid, user_pass)
    except KeyboardInterrupt:
        pass


if __name__ == "__main__":
    main()
